@@ -1,4 +1,5 @@
 use std::{
+    fmt::Display,
     net::{IpAddr, SocketAddr},
     num::{NonZeroU32, NonZeroU8},
     path::PathBuf,
@@ -6,8 +7,10 @@ use std::{
 
 use serde::{Deserialize, Serialize};
 
-use crate::track::MediaType;
+use crate::track::MediaTypesSimple;
+use backends::BackendSettings;
 
+pub mod backends;
 /// Extra things necessary for a config file, like wrappers for versioning
 pub mod config_extra;
 
@@ -20,6 +23,7 @@ pub struct ServerSettings {
     pub com: ComSettings,
     pub player: PlayerSettings,
     pub podcast: PodcastSettings,
+    pub backends: BackendSettings,
 }
 
 #[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq)]
@@ -156,13 +160,13 @@ pub enum PositionYesNo {
 impl PositionYesNo {
     /// Get the time before saving the track position, if enabled
     #[must_use]
-    pub fn get_time(&self, media_type: MediaType) -> Option<u64> {
+    pub fn get_time(&self, media_type: MediaTypesSimple) -> Option<u64> {
         match self {
             PositionYesNo::Simple(v) => match v {
                 PositionYesNoLower::Yes => match media_type {
-                    MediaType::Music => Some(DEFAULT_YES_TIME_BEFORE_SAVE_MUSIC),
-                    MediaType::Podcast => Some(DEFAULT_YES_TIME_BEFORE_SAVE_PODCAST),
-                    MediaType::LiveRadio => None,
+                    MediaTypesSimple::Music => Some(DEFAULT_YES_TIME_BEFORE_SAVE_MUSIC),
+                    MediaTypesSimple::Podcast => Some(DEFAULT_YES_TIME_BEFORE_SAVE_PODCAST),
+                    MediaTypesSimple::LiveRadio => None,
                 },
                 PositionYesNoLower::No => None,
             },
@@ -195,30 +199,30 @@ pub enum RememberLastPosition {
 impl RememberLastPosition {
     /// Get the time before saving the track position, if enabled
     #[must_use]
-    pub fn get_time(&self, media_type: MediaType) -> Option<u64> {
+    pub fn get_time(&self, media_type: MediaTypesSimple) -> Option<u64> {
         match self {
             RememberLastPosition::All(v) => v.get_time(media_type),
             RememberLastPosition::Depends { music, podcast } => match media_type {
-                MediaType::Music => music.get_time(media_type),
-                MediaType::Podcast => podcast.get_time(media_type),
-                MediaType::LiveRadio => None,
+                MediaTypesSimple::Music => music.get_time(media_type),
+                MediaTypesSimple::Podcast => podcast.get_time(media_type),
+                MediaTypesSimple::LiveRadio => None,
             },
         }
     }
 
-    /// Get if remembering for the given [`MediaType`] is enabled or not
+    /// Get if remembering for the given [`MediaTypesSimple`] is enabled or not
     ///
     /// use case is in the restore of the last position
-    #[allow(clippy::needless_pass_by_value)] // "MediaType" is a 1-byte copy
+    #[allow(clippy::needless_pass_by_value)] // "MediaTypesSimple" is a 1-byte copy
     #[must_use]
-    pub fn is_enabled_for(&self, media_type: MediaType) -> bool {
+    pub fn is_enabled_for(&self, media_type: MediaTypesSimple) -> bool {
         match self {
             RememberLastPosition::All(v) => v.is_enabled(),
             RememberLastPosition::Depends { music, podcast } => match media_type {
-                MediaType::Music => music.is_enabled(),
-                MediaType::Podcast => podcast.is_enabled(),
+                MediaTypesSimple::Music => music.is_enabled(),
+                MediaTypesSimple::Podcast => podcast.is_enabled(),
                 // liveradio cannot store a position
-                MediaType::LiveRadio => false,
+                MediaTypesSimple::LiveRadio => false,
             },
         }
     }
@@ -268,6 +272,9 @@ pub struct PlayerSettings {
     pub random_track_quantity: NonZeroU32,
     /// Minimal amount of tracks a album needs to have before being chosen for "random album add"
     pub random_album_min_quantity: NonZeroU32,
+
+    /// The backend to use
+    pub backend: Backend,
 }
 
 /// Get the default Music dir, which uses OS-specific paths, or home/Music
@@ -296,10 +303,40 @@ impl Default for PlayerSettings {
 
             random_track_quantity: NonZeroU32::new(20).unwrap(),
             random_album_min_quantity: NonZeroU32::new(5).unwrap(),
+
+            backend: Backend::default(),
         }
     }
 }
 
+#[derive(Debug, Clone, Copy, Deserialize, Serialize, Default, PartialEq, Eq)]
+#[serde(rename_all = "lowercase")]
+pub enum Backend {
+    #[serde(rename = "gst")]
+    #[serde(alias = "gstreamer")]
+    Gstreamer,
+    Mpv,
+    #[default]
+    Rusty,
+}
+
+impl Backend {
+    #[must_use]
+    pub fn as_str(self) -> &'static str {
+        match self {
+            Backend::Gstreamer => "gst",
+            Backend::Mpv => "mpv",
+            Backend::Rusty => "rusty",
+        }
+    }
+}
+
+impl Display for Backend {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", self.as_str())
+    }
+}
+
 /// Playlist loop modes
 #[derive(Debug, Clone, Copy, Deserialize, Serialize, Default, PartialEq, Eq)]
 #[serde(rename_all = "lowercase")]
@@ -350,11 +387,66 @@ impl LoopMode {
     }
 }
 
-/// Settings for the gRPC server (and potentially future ways to communicate)
+/// Error for when [`ComProtocol`] parsing fails
+#[derive(Debug, Clone, PartialEq, thiserror::Error)]
+pub enum ComProtocolParseError {
+    #[error("Expected \"uds\" or \"http\", found \"{0}\"")]
+    UnknownValue(String),
+}
+
+/// The Protocol to use for the server-client communication
 #[derive(Debug, Clone, Copy, Deserialize, Serialize, PartialEq, Eq)]
+#[serde(try_from = "String")]
+#[serde(into = "String")]
+pub enum ComProtocol {
+    HTTP,
+    /// Unix socket
+    UDS,
+}
+
+impl TryFrom<String> for ComProtocol {
+    type Error = ComProtocolParseError;
+
+    fn try_from(value: String) -> Result<Self, Self::Error> {
+        Self::try_from(value.as_str())
+    }
+}
+
+impl TryFrom<&str> for ComProtocol {
+    type Error = ComProtocolParseError;
+
+    fn try_from(value: &str) -> Result<Self, Self::Error> {
+        let lowercase = value.to_ascii_lowercase();
+        Ok(match lowercase.as_str() {
+            "http" => Self::HTTP,
+            "uds" => Self::UDS,
+            _ => return Err(ComProtocolParseError::UnknownValue(lowercase)),
+        })
+    }
+}
+
+impl From<ComProtocol> for String {
+    fn from(val: ComProtocol) -> Self {
+        match val {
+            ComProtocol::HTTP => "http",
+            ComProtocol::UDS => "uds",
+        }
+        .to_string()
+    }
+}
+
+/// Settings for the gRPC server (and potentially future ways to communicate)
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq)]
 // for now, require that both port and ip are specified at once
-// #[serde(default)] // allow missing fields and fill them with the `..Self::default()` in this struct
+#[serde(default)] // allow missing fields and fill them with the `..Self::default()` in this struct
 pub struct ComSettings {
+    // General Settings
+    pub protocol: ComProtocol,
+
+    // UDS settings
+    pub socket_path: PathBuf,
+
+    // Below are HTTP settings
     /// gRPC server Port
     pub port: u16,
     /// gRPC server interface / address
@@ -363,15 +455,25 @@ pub struct ComSettings {
 
 impl Default for ComSettings {
     fn default() -> Self {
+        // TODO: maybe default to include user id like "termusic-1000.socket"?
+        let socket_path = std::env::temp_dir().join("termusic.socket");
+
         Self {
+            #[cfg(unix)]
+            protocol: ComProtocol::UDS,
+            #[cfg(not(unix))]
+            protocol: ComProtocol::HTTP,
+
+            socket_path,
+
             port: 50101,
             address: "::1".parse().unwrap(),
         }
     }
 }
 
-impl From<ComSettings> for SocketAddr {
-    fn from(value: ComSettings) -> Self {
+impl From<&ComSettings> for SocketAddr {
+    fn from(value: &ComSettings) -> Self {
         Self::new(value.address, value.port)
     }
 }
@@ -380,9 +482,9 @@ mod v1_interop {
     use std::num::TryFromIntError;
 
     use super::{
-        ComSettings, LoopMode, NonZeroU32, NonZeroU8, PlayerSettings, PodcastSettings,
-        PositionYesNo, PositionYesNoLower, RememberLastPosition, ScanDepth, SeekStep,
-        ServerSettings,
+        backends::BackendSettings, Backend, ComSettings, LoopMode, NonZeroU32, NonZeroU8,
+        PlayerSettings, PodcastSettings, PositionYesNo, PositionYesNoLower, RememberLastPosition,
+        ScanDepth, SeekStep, ServerSettings,
     };
     use crate::config::v1;
 
@@ -442,8 +544,11 @@ mod v1_interop {
         #[allow(clippy::cast_possible_truncation)] // checked casts
         fn try_from(value: v1::Settings) -> Result<Self, Self::Error> {
             let com_settings = ComSettings {
+                // when coming from v1, continue using http until manually changed
+                protocol: super::ComProtocol::HTTP,
                 port: value.player_port,
                 address: value.player_interface,
+                ..Default::default()
             };
 
             let podcast_settings = PodcastSettings {
@@ -492,12 +597,15 @@ mod v1_interop {
                     new_key: "player.random_album_min_quantity",
                     source: err,
                 })?,
+
+                backend: Backend::default(),
             };
 
             Ok(Self {
                 com: com_settings,
                 player: player_settings,
                 podcast: podcast_settings,
+                backends: BackendSettings::default(),
             })
         }
     }
@@ -507,6 +615,8 @@ mod v1_interop {
         use pretty_assertions::assert_eq;
         use std::path::PathBuf;
 
+        use crate::config::v2::server::ComProtocol;
+
         use super::*;
 
         #[test]
@@ -532,8 +642,10 @@ mod v1_interop {
             assert_eq!(
                 converted.com,
                 ComSettings {
+                    protocol: ComProtocol::HTTP,
                     port: 50101,
-                    address: "::1".parse().unwrap()
+                    address: "::1".parse().unwrap(),
+                    ..Default::default()
                 }
             );
 
@@ -567,6 +679,7 @@ mod v1_interop {
                     set_discord_status: true,
                     random_track_quantity: NonZeroU32::new(20).unwrap(),
                     random_album_min_quantity: NonZeroU32::new(5).unwrap(),
+                    backend: Backend::default(),
                 }
             );
         }
