@@ -1,3 +1,5 @@
+use crate::songtag::UrlTypes;
+
 /**
  * MIT License
  *
@@ -22,75 +24,166 @@
  * SOFTWARE.
  */
 use super::super::{ServiceProvider, SongTag};
-use serde_json::{json, Value};
+use serde_json::{from_str, json, Value};
 
-pub fn to_lyric(json: &str) -> Option<String> {
-    if let Ok(value) = serde_json::from_str::<Value>(json) {
-        if value.get("msg")?.eq("\u{6210}\u{529f}") {
-            // if value.get("msg")?.eq("成功") {
-            let lyric = value.get("lyric")?.as_str()?.to_owned();
-            return Some(lyric);
-        }
-    }
-    None
+#[derive(Debug, thiserror::Error)]
+pub enum MiguParseError {
+    #[error("Expected field \"{field}\" to have value \"{expected}\", got \"{got}\", error message: \"{errmsg:#?}\"")]
+    UnexpectedStatus {
+        field: &'static str,
+        got: String,
+        errmsg: Option<String>,
+        expected: &'static str,
+    },
+
+    #[error("Expected property \"{0}\" to exist")]
+    MissingProperty(&'static str),
+
+    #[error(transparent)]
+    ParseError(#[from] serde_json::Error),
 }
 
-pub fn to_pic_url(json: &str) -> Option<String> {
-    if let Ok(value) = serde_json::from_str::<Value>(json) {
-        if value.get("msg")?.eq("\u{6210}\u{529f}") {
-            // if value.get("msg")?.eq("成功") {
-            let pic_url = value.get("largePic")?.as_str()?.to_owned();
-            return Some(pic_url);
-        }
-    }
-    None
+type Result<T> = std::result::Result<T, MiguParseError>;
+
+/// Get the given `field`, otherwise return as [`MissingProperty`](MiguParseError::MissingProperty) Error
+fn get_code_prop<'a>(value: &'a Value, field: &'static str) -> Result<&'a Value> {
+    let Some(value) = value.get(field) else {
+        return Err(MiguParseError::MissingProperty(field));
+    };
+
+    Ok(value)
 }
 
-pub fn to_song_info(json: &str) -> Option<Vec<SongTag>> {
-    if let Ok(value) = serde_json::from_str::<Value>(json) {
-        if value.get("success")?.eq(&true) {
-            let mut vec: Vec<SongTag> = Vec::new();
-            let list = json!([]);
-            let array = value.get("musics").unwrap_or(&list).as_array()?;
-            for v in array {
-                if let Some(item) = parse_song_info(v) {
-                    vec.push(item);
-                }
-            }
-            return Some(vec);
-        }
+/// Check the `msg` property for if there was a error
+fn check_msg(value: &Value) -> Result<()> {
+    let msg = get_code_prop(value, "msg")?;
+
+    // english for the chinese characters: "success"
+    if !msg.eq(&"成功") {
+        let message = msg.to_string();
+
+        return Err(MiguParseError::UnexpectedStatus {
+            field: "msg",
+            got: message,
+            errmsg: None,
+            expected: "成功",
+        });
     }
-    None
+
+    Ok(())
 }
 
+/// Check the `success` property for if there was a error
+fn check_success(value: &Value) -> Result<()> {
+    let success = get_code_prop(value, "success")?;
+
+    if !success.eq(&true) {
+        let message = success.to_string();
+
+        return Err(MiguParseError::UnexpectedStatus {
+            field: "success",
+            got: message,
+            errmsg: None,
+            expected: "true",
+        });
+    }
+
+    Ok(())
+}
+
+/// Try to get the lyric lrc content from the given result
+pub fn to_lyric(json: &str) -> Result<String> {
+    let value = from_str::<Value>(json)?;
+
+    check_msg(&value)?;
+
+    let lyric = value
+        .get("lyric")
+        .and_then(Value::as_str)
+        .ok_or(MiguParseError::MissingProperty("lyric"))?
+        .to_owned();
+
+    Ok(lyric)
+}
+
+/// Try to get the picture url from the json response
+pub fn to_pic_url(json: &str) -> Result<String> {
+    let value = from_str::<Value>(json)?;
+
+    check_msg(&value)?;
+
+    let pic_url = value
+        .get("largePic")
+        .and_then(Value::as_str)
+        .ok_or(MiguParseError::MissingProperty("largePic"))?
+        .to_owned();
+
+    Ok(pic_url)
+}
+
+/// Try to get individual [`SongTag`]s from the json response
+pub fn to_song_info(json: &str) -> Result<Vec<SongTag>> {
+    let value = from_str::<Value>(json)?;
+
+    check_success(&value)?;
+
+    let array = value
+        .get("musics")
+        .and_then(Value::as_array)
+        .ok_or(MiguParseError::MissingProperty("musics"))?;
+
+    let mut vec: Vec<SongTag> = Vec::new();
+
+    for elem in array {
+        if let Some(parsed) = parse_song_info(elem) {
+            vec.push(parsed);
+        }
+    }
+
+    Ok(vec)
+}
+
+/// Try to parse a single [`SongTag`] from a given migu value
 fn parse_song_info(v: &Value) -> Option<SongTag> {
+    // not using "Value::to_string()" as that produces a escaped string
+
     let pic_id = v
         .get("cover")
-        .unwrap_or(&json!("N/A"))
-        .as_str()
-        .unwrap_or("")
-        .to_owned();
+        .and_then(Value::as_str)
+        .map(ToOwned::to_owned);
     let artist = v
         .get("singerName")
-        .unwrap_or(&json!("Unknown Singer"))
-        .as_str()
-        .unwrap_or("Unknown Singer")
-        .to_owned();
-    let title = v.get("songName")?.as_str()?.to_owned();
+        .and_then(Value::as_str)
+        .map(ToOwned::to_owned);
+    let title = v
+        .get("songName")
+        .and_then(Value::as_str)
+        .map(ToOwned::to_owned);
 
-    let album_id = v.get("albumId")?.as_str()?.to_owned();
+    let album_id = v
+        .get("albumId")
+        .and_then(Value::as_str)
+        .map(ToOwned::to_owned);
 
     let url = v
         .get("mp3")
-        .unwrap_or(&json!("N/A"))
-        .as_str()
-        .unwrap_or("Copyright protected")
-        .to_owned();
+        .and_then(Value::as_str)
+        .map_or(UrlTypes::Protected, |v| {
+            UrlTypes::FreeDownloadable(v.to_owned())
+        });
+
+    let lyric_id = v
+        .get("copyrightId")
+        .and_then(Value::as_str)
+        .map(ToOwned::to_owned);
+
+    // a songid is always required
+    let song_id = v.get("id").and_then(Value::as_str).map(ToOwned::to_owned)?;
 
     Some(SongTag {
-        song_id: Some(v.get("id")?.as_str()?.to_owned()),
-        title: Some(title),
-        artist: Some(artist),
+        song_id,
+        title,
+        artist,
         album: Some(
             v.get("albumName")
                 .unwrap_or(&json!("Unknown Album"))
@@ -98,18 +191,19 @@ fn parse_song_info(v: &Value) -> Option<SongTag> {
                 .unwrap_or("")
                 .to_owned(),
         ),
-        pic_id: Some(pic_id),
+        pic_id,
         lang_ext: Some("migu".to_string()),
-        service_provider: Some(ServiceProvider::Migu),
-        lyric_id: Some(v.get("copyrightId")?.as_str()?.to_owned()),
+        service_provider: ServiceProvider::Migu,
+        lyric_id,
         url: Some(url),
-        album_id: Some(album_id),
+        album_id,
     })
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
+    use pretty_assertions::assert_eq;
 
     #[test]
     fn should_parse_songinfo() {
@@ -185,10 +279,10 @@ mod tests {
                 title: Some("Track A".to_owned()),
                 album: Some("Some Album 1".to_owned()),
                 lang_ext: Some("migu".to_string()),
-                service_provider: Some(ServiceProvider::Migu),
-                song_id: Some("0000000002".to_owned()),
+                service_provider: ServiceProvider::Migu,
+                song_id: "0000000002".to_owned(),
                 lyric_id: Some("0000000AAAA".to_owned()),
-                url: Some("https://freetyst.nf.migu.cn/SomeLongPercentFilename.mp3?Key=AAAAAAAAAAAAAAAA&Tim=1111111111111&channelid=01&msisdn=AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA".to_owned()),
+                url: Some(UrlTypes::FreeDownloadable("https://freetyst.nf.migu.cn/SomeLongPercentFilename.mp3?Key=AAAAAAAAAAAAAAAA&Tim=1111111111111&channelid=01&msisdn=AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA".to_owned())),
                 pic_id: Some("https://mcontent.migu.cn/newlv2/new/album/20230810/0000000000/someRandomCode.jpg".to_owned()),
                 album_id: Some("0000000000".to_owned())
             }
@@ -201,10 +295,10 @@ mod tests {
                 title: Some("Track B".to_owned()),
                 album: Some("Some Album 2".to_owned()),
                 lang_ext: Some("migu".to_string()),
-                service_provider: Some(ServiceProvider::Migu),
-                song_id: Some("1111111112".to_owned()),
+                service_provider: ServiceProvider::Migu,
+                song_id: "1111111112".to_owned(),
                 lyric_id: Some("1111111BBBB".to_owned()),
-                url: Some("https://freetyst.nf.migu.cn/SomeOtherLongPercentFilename.mp3?Key=AAAAAAAAAAAAAAAA&Tim=1111111111111&channelid=01&msisdn=AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA".to_owned()),
+                url: Some(UrlTypes::FreeDownloadable("https://freetyst.nf.migu.cn/SomeOtherLongPercentFilename.mp3?Key=AAAAAAAAAAAAAAAA&Tim=1111111111111&channelid=01&msisdn=AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA".to_owned())),
                 pic_id: Some("https://tyqk.migu.cn/files/resize/album/2023-12-19/someOtherRandomCode.jpg?200x200".to_owned()),
                 album_id: Some("1111111111".to_owned())
             }
