@@ -1,7 +1,10 @@
+use std::path::PathBuf;
 use std::sync::Arc;
 
 use crate::config::v2::tui::{keys::KeyBinding, theme::styles::ColorTermusic};
+use crate::ids::{IdConfigEditor, IdKey};
 use crate::invidious::{Instance, YoutubeVideo};
+use crate::library_db::SearchCriteria;
 use crate::podcast::{EpData, PodcastFeed, PodcastNoId};
 use crate::songtag::SongTag;
 use anyhow::{anyhow, Result};
@@ -20,32 +23,21 @@ pub enum Msg {
     GeneralSearch(GSMsg),
     HelpPopupShow,
     HelpPopupClose,
-    LayoutTreeView,
-    LayoutDataBase,
-    LayoutPodCast,
+    Layout(MainLayoutMsg),
     Library(LIMsg),
     LyricMessage(LyricMsg),
     LyricCycle,
     LyricAdjustDelay(i64),
-    PlayerToggleGapless,
-    PlayerTogglePause,
-    PlayerVolumeUp,
-    PlayerVolumeDown,
-    PlayerSpeedUp,
-    PlayerSpeedDown,
-    PlayerSeekForward,
-    PlayerSeekBackward,
+    Player(PlayerMsg),
     Playlist(PLMsg),
     Podcast(PCMsg),
+    /// Closes the Quit Popup, if it was shown without quitting.
     QuitPopupCloseCancel,
+    /// Always will directly quit.
     QuitPopupCloseOk,
+    /// Either shows the Quit Dialog if enabled, or if dialog is disabled, directly quits
     QuitPopupShow,
-    SavePlaylistPopupShow,
-    SavePlaylistPopupCloseCancel,
-    SavePlaylistPopupUpdate(String),
-    SavePlaylistPopupCloseOk(String),
-    SavePlaylistConfirmCloseCancel,
-    SavePlaylistConfirmCloseOk(String),
+    SavePlaylist(SavePlaylistMsg),
     TagEditor(TEMsg),
     UpdatePhoto,
     YoutubeSearch(YSMsg),
@@ -59,6 +51,40 @@ pub enum Msg {
     ForceRedraw,
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum MainLayoutMsg {
+    /// Switch to the Music library view
+    TreeView,
+    /// Switch to the Database view
+    DataBase,
+    /// Switch to the Podcast view
+    Podcast,
+}
+
+/// Player relates messages
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum PlayerMsg {
+    ToggleGapless,
+    TogglePause,
+    VolumeUp,
+    VolumeDown,
+    SpeedUp,
+    SpeedDown,
+    SeekForward,
+    SeekBackward,
+}
+
+/// Save Playlist Popup related messages
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub enum SavePlaylistMsg {
+    PopupShow,
+    PopupCloseCancel,
+    PopupUpdate(String),
+    PopupCloseOk(String),
+    ConfirmCloseCancel,
+    ConfirmCloseOk(String),
+}
+
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub enum XYWHMsg {
     /// Toggle the hidden / shown status of the displayed image.
@@ -142,6 +168,8 @@ pub enum ConfigEditorMsg {
     ConfigSaveCancel,
     ExitConfirmationBlurDown,
     ExitConfirmationBlurUp,
+    ExtraYtdlpArgsBlurDown,
+    ExtraYtdlpArgsBlurUp,
     Open,
     KeyFocus(KFMsg),
     MusicDirBlurDown,
@@ -361,6 +389,14 @@ pub enum KFMsg {
     PodcastRefreshAllFeedsBlurUp,
 }
 
+/// Basically a Tree Node, but without having to include `tui-realm-treeview` as another dependency for lib
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct RecVec<T, V> {
+    pub id: T,
+    pub value: V,
+    pub children: Vec<RecVec<T, V>>,
+}
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum LIMsg {
     TreeStepInto(String),
@@ -371,20 +407,35 @@ pub enum LIMsg {
     SwitchRoot,
     AddRoot,
     RemoveRoot,
+
+    /// A requested node is ready from loading.
+    /// `(Tree, FocusNode)`
+    TreeNodeReady(RecVec<PathBuf, String>, Option<String>),
 }
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum DBMsg {
+    /// Add all Track Results (from view `Tracks`) to the playlist
     AddAllToPlaylist,
+    /// Add a single Track Result (from view `Tracks`) to the playlist
     AddPlaylist(usize),
+    /// Add all Results (from view `Result`) to the playlist
+    AddAllResultsToPlaylist,
+    /// Add a single result (from view `Result`) to the playlist
+    AddResultToPlaylist(usize),
     CriteriaBlurDown,
     CriteriaBlurUp,
-    SearchResult(usize),
+    /// Search Results (for view `Result`) from a `Database`(view) index
+    SearchResult(SearchCriteria),
     SearchResultBlurDown,
     SearchResultBlurUp,
+    /// Serarch Tracks (for view `Tracks`) from a `Result`(view) index
     SearchTrack(usize),
     SearchTracksBlurDown,
     SearchTracksBlurUp,
+
+    AddAllResultsConfirmShow,
+    AddAllResultsConfirmCancel,
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -436,7 +487,7 @@ pub enum PLMsg {
     /// Change focus to the previous view
     PlaylistTableBlurUp,
     /// Add a directory / file to the playlist
-    Add(String),
+    Add(PathBuf),
     /// Remove INDEX from playlist
     Delete(usize),
     /// Clear the Playlist
@@ -503,7 +554,7 @@ pub enum YSMsg {
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum TEMsg {
     TagEditorRun(String),
-    TagEditorClose(Option<String>),
+    TagEditorClose,
     TECounterDeleteOk,
     TEDownload(usize),
     TEEmbed(usize),
@@ -511,6 +562,8 @@ pub enum TEMsg {
     TERename,
     TESearch,
     TESelectLyricOk(usize),
+
+    TESearchLyricResult(SongTagRecordingResult),
 }
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
@@ -533,186 +586,8 @@ pub enum TFMsg {
     TextareaLyricBlurUp,
 }
 
-#[derive(Debug, Eq, PartialEq, Clone, Copy, Hash)]
-pub enum Id {
-    ConfigEditor(IdConfigEditor),
-    DBListCriteria,
-    DBListSearchResult,
-    DBListSearchTracks,
-    DeleteConfirmRadioPopup,
-    DeleteConfirmInputPopup,
-    DownloadSpinner,
-    Episode,
-    ErrorPopup,
-    GeneralSearchInput,
-    GeneralSearchTable,
-    GlobalListener,
-    HelpPopup,
-    Label,
-    Library,
-    Lyric,
-    MessagePopup,
-    Playlist,
-    Podcast,
-    PodcastAddPopup,
-    PodcastSearchTablePopup,
-    FeedDeleteConfirmRadioPopup,
-    FeedDeleteConfirmInputPopup,
-    Progress,
-    QuitPopup,
-    SavePlaylistPopup,
-    SavePlaylistLabel,
-    SavePlaylistConfirm,
-    TagEditor(IdTagEditor),
-    YoutubeSearchInputPopup,
-    YoutubeSearchTablePopup,
-}
-
-#[derive(Debug, Eq, PartialEq, Clone, Copy, Hash)]
-pub enum IdTagEditor {
-    CounterDelete,
-    LabelHint,
-    InputArtist,
-    InputTitle,
-    InputAlbum,
-    InputGenre,
-    SelectLyric,
-    TableLyricOptions,
-    TextareaLyric,
-}
-
-#[derive(Debug, Eq, PartialEq, Clone, Copy, Hash)]
-pub enum IdConfigEditor {
-    AlbumPhotoAlign,
-    CEThemeSelect,
-    ConfigSavePopup,
-    ExitConfirmation,
-    Footer,
-    Header,
-    Key(IdKey),
-    KillDamon,
-
-    LibraryBackground,
-    LibraryBorder,
-    LibraryForeground,
-    LibraryHighlight,
-    LibraryHighlightSymbol,
-    LibraryLabel,
-
-    LyricBackground,
-    LyricBorder,
-    LyricForeground,
-    LyricLabel,
-
-    MusicDir,
-    PlayerPort,
-    PlayerUseDiscord,
-    PlayerUseMpris,
-
-    PlaylistBackground,
-    PlaylistBorder,
-    PlaylistDisplaySymbol,
-    PlaylistForeground,
-    PlaylistHighlight,
-    PlaylistHighlightSymbol,
-    PlaylistLabel,
-    PlaylistRandomAlbum,
-    PlaylistRandomTrack,
-
-    CurrentlyPlayingTrackSymbol,
-
-    PodcastDir,
-    PodcastMaxRetries,
-    PodcastSimulDownload,
-
-    ProgressBackground,
-    ProgressBorder,
-    ProgressForeground,
-    ProgressLabel,
-
-    SaveLastPosition,
-    SeekStep,
-
-    ImportantPopupLabel,
-    ImportantPopupBackground,
-    ImportantPopupBorder,
-    ImportantPopupForeground,
-
-    FallbackBackground,
-    FallbackBorder,
-    FallbackForeground,
-    FallbackHighlight,
-    FallbackLabel,
-}
-
-#[derive(Debug, Eq, PartialEq, Clone, Copy, Hash)]
-pub enum IdKey {
-    DatabaseAddAll,
-    DatabaseAddSelected,
-    GlobalConfig,
-    GlobalDown,
-    GlobalGotoBottom,
-    GlobalGotoTop,
-    GlobalHelp,
-    GlobalLayoutTreeview,
-    GlobalLayoutDatabase,
-    GlobalLeft,
-    GlobalLyricAdjustForward,
-    GlobalLyricAdjustBackward,
-    GlobalLyricCycle,
-    GlobalPlayerToggleGapless,
-    GlobalPlayerTogglePause,
-    GlobalPlayerNext,
-    GlobalPlayerPrevious,
-    GlobalPlayerSeekForward,
-    GlobalPlayerSeekBackward,
-    GlobalPlayerSpeedUp,
-    GlobalPlayerSpeedDown,
-    GlobalQuit,
-    GlobalRight,
-    GlobalUp,
-    GlobalVolumeDown,
-    GlobalVolumeUp,
-    GlobalSavePlaylist,
-    LibraryDelete,
-    LibraryLoadDir,
-    LibraryPaste,
-    LibrarySearch,
-    LibrarySearchYoutube,
-    LibraryTagEditor,
-    LibraryYank,
-    PlaylistDelete,
-    PlaylistDeleteAll,
-    PlaylistShuffle,
-    PlaylistModeCycle,
-    PlaylistPlaySelected,
-    PlaylistSearch,
-    PlaylistSwapDown,
-    PlaylistSwapUp,
-    PlaylistAddRandomAlbum,
-    PlaylistAddRandomTracks,
-    LibrarySwitchRoot,
-    LibraryAddRoot,
-    LibraryRemoveRoot,
-    GlobalLayoutPodcast,
-    GlobalXywhMoveLeft,
-    GlobalXywhMoveRight,
-    GlobalXywhMoveUp,
-    GlobalXywhMoveDown,
-    GlobalXywhZoomIn,
-    GlobalXywhZoomOut,
-    GlobalXywhHide,
-    PodcastMarkPlayed,
-    PodcastMarkAllPlayed,
-    PodcastEpDownload,
-    PodcastEpDeleteFile,
-    PodcastDeleteFeed,
-    PodcastDeleteAllFeeds,
-    PodcastSearchAddFeed,
-    PodcastRefreshFeed,
-    PodcastRefreshAllFeeds,
-}
-pub enum SearchLyricState {
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub enum SongTagRecordingResult {
     Finish(Vec<SongTag>),
 }
 
