@@ -1,80 +1,45 @@
-/*
- * MIT License
- *
- * termusic - Copyright (c) 2021 Larry Hao
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in all
- * copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
- */
+//! SPDX-License-Identifier: MIT
 
-#[cfg(feature = "gst")]
-mod gstreamer_backend;
-#[cfg(feature = "mpv")]
-mod mpv_backend;
-mod rusty_backend;
-
-mod discord;
-mod mpris;
-pub mod playlist;
+use std::sync::Arc;
+use std::time::Duration;
 
 use anyhow::{Context, Result};
 use async_trait::async_trait;
-pub use playlist::{Playlist, Status};
-use std::time::Duration;
+use parking_lot::RwLock;
+pub use playlist::Playlist;
 use termusiclib::config::v2::server::config_extra::ServerConfigVersionedDefaulted;
-use termusiclib::config::{ServerOverlay, SharedServerSettings};
+use termusiclib::config::SharedServerSettings;
 use termusiclib::library_db::DataBase;
-use termusiclib::player::{PlayerProgress, PlayerTimeUnit, TrackChangedInfo, UpdateEvents};
+use termusiclib::player::playlist_helpers::{
+    PlaylistAddTrack, PlaylistPlaySpecific, PlaylistRemoveTrackIndexed, PlaylistSwapTrack,
+};
+use termusiclib::player::{
+    PlayerProgress, PlayerTimeUnit, RunningStatus, TrackChangedInfo, UpdateEvents,
+};
 use termusiclib::podcast::db::Database as DBPod;
-use termusiclib::track::{MediaType, Track};
+use termusiclib::track::{MediaTypesSimple, Track};
 use termusiclib::utils::get_app_config_path;
 use tokio::runtime::Handle;
 use tokio::sync::mpsc::error::SendError;
 use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender};
 use tokio::sync::{broadcast, oneshot};
 
+pub use backends::{Backend, BackendSelect};
+
+mod discord;
+mod mpris;
+pub mod playlist;
+
 #[macro_use]
 extern crate log;
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Default)]
-pub enum BackendSelect {
-    #[cfg(feature = "mpv")]
-    Mpv,
-    #[cfg(feature = "gst")]
-    GStreamer,
-    /// Create a new Backend with default backend ordering
-    ///
-    /// Order:
-    /// - [`GstreamerBackend`](gstreamer_backend::GStreamerBackend) (feature `gst`)
-    /// - [`MpvBackend`](mpv_backend::MpvBackend) (feature `mpv`)
-    /// - [`RustyBackend`](rusty_backend::RustyBackend) (default)
-    #[default]
-    Rusty,
-}
+mod backends;
 
-/// Enum to choose backend at runtime
-#[non_exhaustive]
-pub enum Backend {
-    #[cfg(feature = "mpv")]
-    Mpv(mpv_backend::MpvBackend),
-    Rusty(rusty_backend::RustyBackend),
-    #[cfg(feature = "gst")]
-    GStreamer(gstreamer_backend::GStreamerBackend),
+/// Private module for benchmarking only, should never be used outside.
+///
+/// This is necessary as benchmarking via criterion can only access public lib(crate) level function, like any other outside binary / crate.
+pub mod __bench {
+    pub use super::backends::rusty::source::async_ring;
 }
 
 pub type PlayerCmdCallback = oneshot::Receiver<()>;
@@ -129,71 +94,12 @@ impl PlayerCmdSender {
     }
 }
 
-impl Backend {
-    /// Create a new Backend based on `backend`([`BackendSelect`])
-    fn new_select(backend: BackendSelect, config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
-        match backend {
-            #[cfg(feature = "mpv")]
-            BackendSelect::Mpv => Self::new_mpv(config, cmd_tx),
-            #[cfg(feature = "gst")]
-            BackendSelect::GStreamer => Self::new_gstreamer(config, cmd_tx),
-            BackendSelect::Rusty => Self::new_rusty(config, cmd_tx),
-        }
-    }
-
-    // /// Create a new Backend with default backend ordering
-    // ///
-    // /// For the order see [`BackendSelect::Default`]
-    // #[allow(unreachable_code)]
-    // fn new_default(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
-    //     #[cfg(feature = "gst")]
-    //     return Self::new_gstreamer(config, cmd_tx);
-    //     #[cfg(feature = "mpv")]
-    //     return Self::new_mpv(config, cmd_tx);
-    //     return Self::new_rusty(config, cmd_tx);
-    // }
-
-    /// Explicitly choose Backend [`RustyBackend`](rusty_backend::RustyBackend)
-    fn new_rusty(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
-        info!("Using Backend \"rusty\"");
-        Self::Rusty(rusty_backend::RustyBackend::new(config, cmd_tx))
-    }
-
-    /// Explicitly choose Backend [`GstreamerBackend`](gstreamer_backend::GStreamerBackend)
-    #[cfg(feature = "gst")]
-    fn new_gstreamer(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
-        info!("Using Backend \"GStreamer\"");
-        Self::GStreamer(gstreamer_backend::GStreamerBackend::new(config, cmd_tx))
-    }
-
-    /// Explicitly choose Backend [`MpvBackend`](mpv_backend::MpvBackend)
-    #[cfg(feature = "mpv")]
-    fn new_mpv(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
-        info!("Using Backend \"mpv\"");
-        Self::Mpv(mpv_backend::MpvBackend::new(config, cmd_tx))
-    }
-
-    #[must_use]
-    pub fn as_player(&self) -> &dyn PlayerTrait {
-        match self {
-            #[cfg(feature = "mpv")]
-            Backend::Mpv(v) => v,
-            #[cfg(feature = "gst")]
-            Backend::GStreamer(v) => v,
-            Backend::Rusty(v) => v,
-        }
-    }
-
-    #[must_use]
-    pub fn as_player_mut(&mut self) -> &mut (dyn PlayerTrait + Send) {
-        match self {
-            #[cfg(feature = "mpv")]
-            Backend::Mpv(v) => v,
-            #[cfg(feature = "gst")]
-            Backend::GStreamer(v) => v,
-            Backend::Rusty(v) => v,
-        }
-    }
+#[derive(Clone, Debug, Copy, PartialEq)]
+pub enum PlayerErrorType {
+    /// The error happened for the currently playing track.
+    Current,
+    /// The error happened for the track that was tried to be enqueued.
+    Enqueue,
 }
 
 #[derive(Clone, Debug)]
@@ -202,7 +108,6 @@ pub enum PlayerCmd {
     CycleLoop,
     Eos,
     GetProgress,
-    PlaySelected,
     SkipPrevious,
     Pause,
     Play,
@@ -219,14 +124,28 @@ pub enum PlayerCmd {
     TogglePause,
     VolumeDown,
     VolumeUp,
+    /// A Error happened in the backend (for example `NotFound`) that makes it unrecoverable to continue to play the current track.
+    /// This will basically be treated as a [`Eos`](PlayerCmd::Eos), with some extra handling.
+    ///
+    /// This should **not** be used if the whole backend is unrecoverable.
+    Error(PlayerErrorType),
+
+    PlaylistPlaySpecific(PlaylistPlaySpecific),
+    PlaylistAddTrack(PlaylistAddTrack),
+    PlaylistRemoveTrack(PlaylistRemoveTrackIndexed),
+    PlaylistClear,
+    PlaylistSwapTrack(PlaylistSwapTrack),
+    PlaylistShuffle,
+    PlaylistRemoveDeletedTracks,
 }
 
 pub type StreamTX = broadcast::Sender<UpdateEvents>;
+pub type SharedPlaylist = Arc<RwLock<Playlist>>;
 
 #[allow(clippy::module_name_repetitions)]
 pub struct GeneralPlayer {
     pub backend: Backend,
-    pub playlist: Playlist,
+    pub playlist: SharedPlaylist,
     pub config: SharedServerSettings,
     pub current_track_updated: bool,
     pub mpris: Option<mpris::Mpris>,
@@ -235,6 +154,9 @@ pub struct GeneralPlayer {
     pub db_podcast: DBPod,
     pub cmd_tx: PlayerCmdSender,
     pub stream_tx: StreamTX,
+
+    /// Keep track of continues backend errors (like `NotFound`) to not keep trying infinitely.
+    pub errors_since_last_progress: usize,
 }
 
 impl GeneralPlayer {
@@ -249,16 +171,16 @@ impl GeneralPlayer {
         config: SharedServerSettings,
         cmd_tx: PlayerCmdSender,
         stream_tx: StreamTX,
+        playlist: SharedPlaylist,
     ) -> Result<Self> {
-        let config_read = config.read();
-        let backend = Backend::new_select(backend, &config_read, cmd_tx.clone());
+        let backend = Backend::new_select(backend, config.clone(), cmd_tx.clone());
 
         let db_path = get_app_config_path().with_context(|| "failed to get podcast db path.")?;
 
         let db_podcast = DBPod::new(&db_path).with_context(|| "error connecting to podcast db.")?;
+        let config_read = config.read();
         let db = DataBase::new(&config_read)?;
 
-        let playlist = Playlist::new(&config).context("Failed to load playlist")?;
         let mpris = if config.read().settings.player.use_mediacontrols {
             Some(mpris::Mpris::new(cmd_tx.clone()))
         } else {
@@ -283,10 +205,22 @@ impl GeneralPlayer {
             cmd_tx,
             stream_tx,
             current_track_updated: false,
+
+            errors_since_last_progress: 0,
         })
     }
 
-    /// Create a new [`GeneralPlayer`], with the [`BackendSelect::Default`] backend
+    /// Increment the errors that happened by one.
+    pub fn increment_errors(&mut self) {
+        self.errors_since_last_progress += 1;
+    }
+
+    /// Reset errors that happened back to 0
+    pub fn reset_errors(&mut self) {
+        self.errors_since_last_progress = 0;
+    }
+
+    /// Create a new [`GeneralPlayer`], with the default Backend ([`BackendSelect::Rusty`])
     ///
     /// # Errors
     ///
@@ -296,8 +230,15 @@ impl GeneralPlayer {
         config: SharedServerSettings,
         cmd_tx: PlayerCmdSender,
         stream_tx: StreamTX,
+        playlist: SharedPlaylist,
     ) -> Result<Self> {
-        Self::new_backend(BackendSelect::Rusty, config, cmd_tx, stream_tx)
+        Self::new_backend(
+            BackendSelect::default(),
+            config,
+            cmd_tx,
+            stream_tx,
+            playlist,
+        )
     }
 
     /// Reload the config from file, on fail continue to use the old
@@ -315,7 +256,7 @@ impl GeneralPlayer {
             // start mpris if new config has it enabled, but is not active yet
             let mut mpris = mpris::Mpris::new(self.cmd_tx.clone());
             // actually set the metadata of the currently playing track, otherwise the controls will work but no title or coverart will be set until next track
-            if let Some(track) = self.playlist.current_track() {
+            if let Some(track) = self.playlist.read().current_track() {
                 mpris.add_and_play(track);
             }
             // the same for volume
@@ -331,7 +272,7 @@ impl GeneralPlayer {
             let discord = discord::Rpc::default();
 
             // actually set the metadata of the currently playing track, otherwise the controls will work but no title or coverart will be set until next track
-            if let Some(track) = self.playlist.current_track() {
+            if let Some(track) = self.playlist.read().current_track() {
                 discord.update(track);
             }
 
@@ -367,26 +308,25 @@ impl GeneralPlayer {
     ///
     /// if `current_track_index` in playlist is above u32
     pub fn start_play(&mut self) {
-        if self.playlist.is_stopped() | self.playlist.is_paused() {
-            self.playlist.set_status(Status::Running);
+        let mut playlist = self.playlist.write();
+        if playlist.is_stopped() | playlist.is_paused() {
+            playlist.set_status(RunningStatus::Running);
         }
 
-        self.playlist.proceed();
+        playlist.proceed();
 
-        if let Some(track) = self.playlist.current_track().cloned() {
-            info!("Starting Track {:#?}", track);
+        if let Some(track) = playlist.current_track().cloned() {
+            info!("Starting Track {track:#?}");
 
-            if self.playlist.has_next_track() {
-                self.playlist.set_next_track(None);
+            if playlist.has_next_track() {
+                playlist.set_next_track(None);
+                drop(playlist);
                 self.current_track_updated = true;
                 info!("gapless next track played");
-                #[allow(irrefutable_let_patterns)]
-                if let Backend::Rusty(ref mut backend) = self.backend {
-                    backend.message_on_end();
-                }
                 self.add_and_play_mpris_discord();
                 return;
             }
+            drop(playlist);
 
             self.current_track_updated = true;
             let wait = async {
@@ -396,13 +336,9 @@ impl GeneralPlayer {
 
             self.add_and_play_mpris_discord();
             self.player_restore_last_position();
-            #[allow(irrefutable_let_patterns)]
-            if let Backend::Rusty(ref mut backend) = self.backend {
-                backend.message_on_end();
-            }
 
             self.send_stream_ev(UpdateEvents::TrackChanged(TrackChangedInfo {
-                current_track_index: u64::try_from(self.playlist.get_current_track_index())
+                current_track_index: u64::try_from(self.playlist.read().get_current_track_index())
                     .unwrap(),
                 current_track_updated: self.current_track_updated,
                 title: self.media_info().media_title,
@@ -412,7 +348,7 @@ impl GeneralPlayer {
     }
 
     fn add_and_play_mpris_discord(&mut self) {
-        if let Some(track) = self.playlist.current_track() {
+        if let Some(track) = self.playlist.read().current_track() {
             if let Some(ref mut mpris) = self.mpris {
                 mpris.add_and_play(track);
             }
@@ -423,24 +359,26 @@ impl GeneralPlayer {
         }
     }
     pub fn enqueue_next_from_playlist(&mut self) {
-        if self.playlist.has_next_track() {
+        let mut playlist = self.playlist.write();
+        if playlist.has_next_track() {
             return;
         }
 
-        let Some(track) = self.playlist.fetch_next_track().cloned() else {
+        let Some(track) = playlist.fetch_next_track().cloned() else {
             return;
         };
+        drop(playlist);
 
         self.enqueue_next(&track);
 
-        info!("Next track enqueued: {:#?}", track);
+        info!("Next track enqueued: {track:#?}");
     }
 
     /// Skip to the next track, if there is one
     pub fn next(&mut self) {
-        if self.playlist.current_track().is_some() {
+        if self.playlist.read().current_track().is_some() {
             info!("skip route 1 which is in most cases.");
-            self.playlist.set_next_track(None);
+            self.playlist.write().set_next_track(None);
             self.skip_one();
         } else {
             info!("skip route 2 cause no current track.");
@@ -450,19 +388,24 @@ impl GeneralPlayer {
 
     /// Switch & Play the previous track in the playlist
     pub fn previous(&mut self) {
-        self.playlist.previous();
-        self.playlist.proceed_false();
+        let mut playlist = self.playlist.write();
+        playlist.previous();
+        playlist.proceed_false();
+        drop(playlist);
         self.next();
     }
 
     /// Resume playback if paused, pause playback if running
     pub fn toggle_pause(&mut self) {
-        match self.playlist.status() {
-            Status::Running => {
+        // NOTE: if this ".read()" call is in a match's statement, it will not be unlocked until the end of the match
+        // see https://github.com/rust-lang/rust/issues/93883
+        let status = self.playlist.read().status();
+        match status {
+            RunningStatus::Running => {
                 <Self as PlayerTrait>::pause(self);
             }
-            Status::Stopped => {}
-            Status::Paused => {
+            RunningStatus::Stopped => {}
+            RunningStatus::Paused => {
                 <Self as PlayerTrait>::resume(self);
             }
         }
@@ -470,19 +413,25 @@ impl GeneralPlayer {
 
     /// Pause playback if running
     pub fn pause(&mut self) {
-        match self.playlist.status() {
-            Status::Running => {
+        // NOTE: if this ".read()" call is in a match's statement, it will not be unlocked until the end of the match
+        // see https://github.com/rust-lang/rust/issues/93883
+        let status = self.playlist.read().status();
+        match status {
+            RunningStatus::Running => {
                 <Self as PlayerTrait>::pause(self);
             }
-            Status::Stopped | Status::Paused => {}
+            RunningStatus::Stopped | RunningStatus::Paused => {}
         }
     }
 
     /// Resume playback if paused
     pub fn play(&mut self) {
-        match self.playlist.status() {
-            Status::Running | Status::Stopped => {}
-            Status::Paused => {
+        // NOTE: if this ".read()" call is in a match's statement, it will not be unlocked until the end of the match
+        // see https://github.com/rust-lang/rust/issues/93883
+        let status = self.playlist.read().status();
+        match status {
+            RunningStatus::Running | RunningStatus::Stopped => {}
+            RunningStatus::Paused => {
                 <Self as PlayerTrait>::resume(self);
             }
         }
@@ -491,12 +440,14 @@ impl GeneralPlayer {
     ///
     /// if the underlying "seek" returns a error (which current never happens)
     pub fn seek_relative(&mut self, forward: bool) {
-        let track_len = if let Some(track) = self.playlist.current_track() {
-            track.duration().as_secs()
-        } else {
-            // fallback to 5 instead of not seeking at all
-            5
-        };
+        // fallback to 5 instead of not seeking at all
+        let track_len = self
+            .playlist
+            .read()
+            .current_track()
+            .and_then(Track::duration)
+            .unwrap_or(Duration::from_secs(5))
+            .as_secs();
 
         let mut offset = self
             .config
@@ -514,7 +465,8 @@ impl GeneralPlayer {
 
     #[allow(clippy::cast_sign_loss)]
     pub fn player_save_last_position(&mut self) {
-        let Some(track) = self.playlist.current_track() else {
+        let playlist = self.playlist.read();
+        let Some(track) = playlist.current_track() else {
             info!("Not saving Last position as there is no current track");
             return;
         };
@@ -529,28 +481,28 @@ impl GeneralPlayer {
             .settings
             .player
             .remember_position
-            .get_time(track.media_type)
+            .get_time(track.media_type())
         else {
             info!(
                 "Not saving Last position as \"Remember last position\" is not enabled for {:#?}",
-                track.media_type
+                track.media_type()
             );
             return;
         };
 
         if time_before_save < position.as_secs() {
-            match track.media_type {
-                MediaType::Music => {
+            match track.media_type() {
+                MediaTypesSimple::Music => {
                     if let Err(err) = self.db.set_last_position(track, position) {
-                        error!("Saving last_position for music failed, Error: {:#?}", err);
+                        error!("Saving last_position for music failed, Error: {err:#?}");
                     }
                 }
-                MediaType::Podcast => {
+                MediaTypesSimple::LiveRadio => (),
+                MediaTypesSimple::Podcast => {
                     if let Err(err) = self.db_podcast.set_last_position(track, position) {
-                        error!("Saving last_position for podcast failed, Error: {:#?}", err);
+                        error!("Saving last_position for podcast failed, Error: {err:#?}");
                     }
                 }
-                MediaType::LiveRadio => (),
             }
         } else {
             info!("Not saving Last position as the position is lower than time_before_save");
@@ -558,10 +510,12 @@ impl GeneralPlayer {
     }
 
     pub fn player_restore_last_position(&mut self) {
-        let Some(track) = self.playlist.current_track() else {
+        let playlist = self.playlist.read();
+        let Some(track) = playlist.current_track().cloned() else {
             info!("Not restoring Last position as there is no current track");
             return;
         };
+        drop(playlist);
 
         let mut restored = false;
 
@@ -571,35 +525,34 @@ impl GeneralPlayer {
             .settings
             .player
             .remember_position
-            .is_enabled_for(track.media_type)
+            .is_enabled_for(track.media_type())
         {
-            match track.media_type {
-                MediaType::Music => {
-                    if let Ok(last_pos) = self.db.get_last_position(track) {
+            match track.media_type() {
+                MediaTypesSimple::Music => {
+                    if let Ok(last_pos) = self.db.get_last_position(&track) {
                         self.seek_to(last_pos);
                         restored = true;
                     }
                 }
-                MediaType::Podcast => {
-                    if let Ok(last_pos) = self.db_podcast.get_last_position(track) {
+                MediaTypesSimple::LiveRadio => (),
+                MediaTypesSimple::Podcast => {
+                    if let Ok(last_pos) = self.db_podcast.get_last_position(&track) {
                         self.seek_to(last_pos);
                         restored = true;
                     }
                 }
-                MediaType::LiveRadio => (),
             }
         } else {
             info!(
                 "Not restoring Last position as it is not enabled for {:#?}",
-                track.media_type
+                track.media_type()
             );
         }
 
         if restored {
-            if let Some(track) = self.playlist.current_track() {
-                if let Err(err) = self.db.set_last_position(track, Duration::from_secs(0)) {
-                    error!("Resetting last_position failed, Error: {:#?}", err);
-                }
+            // TODO: dosnt this apply podcast titles into the tracks database?
+            if let Err(err) = self.db.set_last_position(&track, Duration::from_secs(0)) {
+                error!("Resetting last_position failed, Error: {err:#?}");
             }
         }
     }
@@ -635,7 +588,7 @@ impl PlayerTrait for GeneralPlayer {
     }
     /// This function should not be used directly, use GeneralPlayer::pause
     fn pause(&mut self) {
-        self.playlist.set_status(Status::Paused);
+        self.playlist.write().set_status(RunningStatus::Paused);
         self.get_player_mut().pause();
         if let Some(ref mut mpris) = self.mpris {
             mpris.pause();
@@ -645,12 +598,12 @@ impl PlayerTrait for GeneralPlayer {
         }
 
         self.send_stream_ev(UpdateEvents::PlayStateChanged {
-            playing: Status::Paused.as_u32(),
+            playing: RunningStatus::Paused.as_u32(),
         });
     }
     /// This function should not be used directly, use GeneralPlayer::play
     fn resume(&mut self) {
-        self.playlist.set_status(Status::Running);
+        self.playlist.write().set_status(RunningStatus::Running);
         self.get_player_mut().resume();
         if let Some(ref mut mpris) = self.mpris {
             mpris.resume();
@@ -661,7 +614,7 @@ impl PlayerTrait for GeneralPlayer {
         }
 
         self.send_stream_ev(UpdateEvents::PlayStateChanged {
-            playing: Status::Running.as_u32(),
+            playing: RunningStatus::Running.as_u32(),
         });
     }
     fn is_paused(&self) -> bool {
@@ -693,7 +646,7 @@ impl PlayerTrait for GeneralPlayer {
     }
 
     fn stop(&mut self) {
-        self.playlist.stop();
+        self.playlist.write().stop();
         self.get_player_mut().stop();
     }
 
