@@ -1,4 +1,3 @@
-use anyhow::Result;
 /**
  * MIT License
  *
@@ -22,21 +21,22 @@ use anyhow::Result;
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-use std::convert::From;
+use anyhow::Result;
 use termusiclib::config::v2::tui::theme::styles::ColorTermusic;
 use termusiclib::config::v2::tui::theme::ThemeWrap;
 use termusiclib::config::SharedTuiSettings;
-use termusiclib::types::{ConfigEditorMsg, Id, IdConfigEditor, Msg};
+use termusiclib::ids::{Id, IdConfigEditor};
+use termusiclib::types::{ConfigEditorMsg, Msg};
 use tui_realm_stdlib::{Input, Label, Select, Table};
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
-use tuirealm::event::{Key, KeyEvent, KeyModifiers, NoUserEvent};
+use tuirealm::event::{Key, KeyEvent, KeyModifiers};
 use tuirealm::props::{
     Alignment, BorderType, Borders, Color, InputType, Style, TableBuilder, TextModifiers, TextSpan,
 };
 use tuirealm::ratatui::style::Modifier;
 use tuirealm::{AttrValue, Attribute, Component, Event, MockComponent, State, StateValue};
 
-use crate::ui::model::Model;
+use crate::ui::model::{Model, UserEvent};
 
 const COLOR_LIST: [ColorTermusic; 19] = [
     ColorTermusic::Reset,
@@ -101,8 +101,8 @@ impl CEThemeSelectTable {
     }
 }
 
-impl Component<Msg, NoUserEvent> for CEThemeSelectTable {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for CEThemeSelectTable {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
         let cmd_result = match ev {
@@ -302,8 +302,8 @@ impl CEColorSelect {
     }
 }
 
-impl Component<Msg, NoUserEvent> for CEColorSelect {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for CEColorSelect {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
         let cmd_result = match ev {
@@ -376,8 +376,8 @@ impl Default for ConfigLibraryTitle {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLibraryTitle {
-    fn on(&mut self, _ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLibraryTitle {
+    fn on(&mut self, _ev: Event<UserEvent>) -> Option<Msg> {
         None
     }
 }
@@ -403,8 +403,8 @@ impl ConfigLibraryForeground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLibraryForeground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLibraryForeground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -430,8 +430,8 @@ impl ConfigLibraryBackground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLibraryBackground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLibraryBackground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -457,8 +457,8 @@ impl ConfigLibraryBorder {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLibraryBorder {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLibraryBorder {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -484,8 +484,8 @@ impl ConfigLibraryHighlight {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLibraryHighlight {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLibraryHighlight {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -505,8 +505,8 @@ impl Default for ConfigPlaylistTitle {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigPlaylistTitle {
-    fn on(&mut self, _ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigPlaylistTitle {
+    fn on(&mut self, _ev: Event<UserEvent>) -> Option<Msg> {
         None
     }
 }
@@ -532,8 +532,8 @@ impl ConfigPlaylistForeground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigPlaylistForeground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigPlaylistForeground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -559,8 +559,8 @@ impl ConfigPlaylistBackground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigPlaylistBackground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigPlaylistBackground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -586,8 +586,8 @@ impl ConfigPlaylistBorder {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigPlaylistBorder {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigPlaylistBorder {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -613,8 +613,8 @@ impl ConfigPlaylistHighlight {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigPlaylistHighlight {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigPlaylistHighlight {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -634,8 +634,8 @@ impl Default for ConfigProgressTitle {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigProgressTitle {
-    fn on(&mut self, _ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigProgressTitle {
+    fn on(&mut self, _ev: Event<UserEvent>) -> Option<Msg> {
         None
     }
 }
@@ -661,8 +661,8 @@ impl ConfigProgressForeground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigProgressForeground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigProgressForeground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -688,8 +688,8 @@ impl ConfigProgressBackground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigProgressBackground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigProgressBackground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -715,8 +715,8 @@ impl ConfigProgressBorder {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigProgressBorder {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigProgressBorder {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -736,8 +736,8 @@ impl Default for ConfigLyricTitle {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLyricTitle {
-    fn on(&mut self, _ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLyricTitle {
+    fn on(&mut self, _ev: Event<UserEvent>) -> Option<Msg> {
         None
     }
 }
@@ -763,8 +763,8 @@ impl ConfigLyricForeground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLyricForeground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLyricForeground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -790,8 +790,8 @@ impl ConfigLyricBackground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLyricBackground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLyricBackground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -817,8 +817,8 @@ impl ConfigLyricBorder {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLyricBorder {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLyricBorder {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -915,8 +915,8 @@ impl ConfigInputHighlight {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigInputHighlight {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigInputHighlight {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
         match ev {
@@ -1028,8 +1028,8 @@ impl ConfigLibraryHighlightSymbol {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigLibraryHighlightSymbol {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigLibraryHighlightSymbol {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -1051,8 +1051,8 @@ impl ConfigPlaylistHighlightSymbol {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigPlaylistHighlightSymbol {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigPlaylistHighlightSymbol {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -1074,8 +1074,8 @@ impl ConfigCurrentlyPlayingTrackSymbol {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigCurrentlyPlayingTrackSymbol {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigCurrentlyPlayingTrackSymbol {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -1095,8 +1095,8 @@ impl Default for ConfigImportantPopupTitle {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigImportantPopupTitle {
-    fn on(&mut self, _ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigImportantPopupTitle {
+    fn on(&mut self, _ev: Event<UserEvent>) -> Option<Msg> {
         None
     }
 }
@@ -1122,8 +1122,8 @@ impl ConfigImportantPopupForeground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigImportantPopupForeground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigImportantPopupForeground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -1149,8 +1149,8 @@ impl ConfigImportantPopupBackground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigImportantPopupBackground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigImportantPopupBackground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -1176,8 +1176,8 @@ impl ConfigImportantPopupBorder {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigImportantPopupBorder {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigImportantPopupBorder {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -1197,8 +1197,8 @@ impl Default for ConfigFallbackTitle {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigFallbackTitle {
-    fn on(&mut self, _ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigFallbackTitle {
+    fn on(&mut self, _ev: Event<UserEvent>) -> Option<Msg> {
         None
     }
 }
@@ -1224,8 +1224,8 @@ impl ConfigFallbackForeground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigFallbackForeground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigFallbackForeground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -1251,8 +1251,8 @@ impl ConfigFallbackBackground {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigFallbackBackground {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigFallbackBackground {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -1278,8 +1278,8 @@ impl ConfigFallbackBorder {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigFallbackBorder {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigFallbackBorder {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
@@ -1305,8 +1305,8 @@ impl ConfigFallbackHighlight {
     }
 }
 
-impl Component<Msg, NoUserEvent> for ConfigFallbackHighlight {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for ConfigFallbackHighlight {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         self.component.on(ev)
     }
 }
