@@ -30,9 +30,13 @@ use termusiclib::config::v2::tui::config_extra::TuiConfigVersionedDefaulted;
 use termusiclib::config::v2::tui::keys::KeyBinding;
 use termusiclib::config::v2::tui::theme::styles::ColorTermusic;
 use termusiclib::config::v2::tui::theme::ThemeColors;
-use termusiclib::types::{ConfigEditorMsg, Id, IdConfigEditor, IdKey, KFMsg, Msg};
+use termusiclib::ids::{Id, IdConfigEditor, IdKey};
+use termusiclib::types::{ConfigEditorMsg, KFMsg, Msg};
 use termusiclib::utils::get_app_config_path;
 
+/// How many Themes there are without actual files and always exist
+pub const THEMES_WITHOUT_FILES: usize = 2;
+
 impl Model {
     #[allow(clippy::too_many_lines)]
     pub fn update_config_editor(&mut self, msg: ConfigEditorMsg) -> Option<Msg> {
@@ -57,7 +61,7 @@ impl Model {
             ConfigEditorMsg::ChangeLayout => self.action_change_layout(),
             ConfigEditorMsg::ConfigChanged => self.config_editor.config_changed = true,
             // Handle focus of general page
-            ConfigEditorMsg::PlayerPortBlurDown | ConfigEditorMsg::ExitConfirmationBlurUp => {
+            ConfigEditorMsg::ExtraYtdlpArgsBlurDown | ConfigEditorMsg::ExitConfirmationBlurUp => {
                 self.app
                     .active(&Id::ConfigEditor(IdConfigEditor::MusicDir))
                     .ok();
@@ -139,11 +143,18 @@ impl Model {
                     .ok();
             }
 
-            ConfigEditorMsg::PlayerUseDiscordBlurDown | ConfigEditorMsg::MusicDirBlurUp => {
+            ConfigEditorMsg::PlayerUseDiscordBlurDown | ConfigEditorMsg::ExtraYtdlpArgsBlurUp => {
                 self.app
                     .active(&Id::ConfigEditor(IdConfigEditor::PlayerPort))
                     .ok();
             }
+
+            ConfigEditorMsg::PlayerPortBlurDown | ConfigEditorMsg::MusicDirBlurUp => {
+                self.app
+                    .active(&Id::ConfigEditor(IdConfigEditor::ExtraYtdlpArgs))
+                    .ok();
+            }
+
             ConfigEditorMsg::ConfigSaveOk => {
                 self.app
                     .umount(&Id::ConfigEditor(IdConfigEditor::ConfigSavePopup))
@@ -359,7 +370,7 @@ impl Model {
                         self.config_editor.theme.style.playlist.current_track_symbol = symbol;
                     }
                     _ => {}
-                };
+                }
             }
 
             ConfigEditorMsg::KeyChange(id, binding) => self.update_key(id, binding),
@@ -376,9 +387,14 @@ impl Model {
 
             return;
         }
+        if index == 1 {
+            self.preview_theme_apply(ThemeColors::full_native(), 1);
 
-        // idx - 1 as 0 table-entry is termusic default, which always exists
-        if let Some(theme_filename) = self.config_editor.themes.get(index - 1) {
+            return;
+        }
+
+        // idx - THEMES_WITHOUT_FILES as 0 until THEMES_WITHOUT_FILES table-entries are termusic themes without files, which always exists
+        if let Some(theme_filename) = self.config_editor.themes.get(index - THEMES_WITHOUT_FILES) {
             match get_app_config_path() {
                 Ok(mut theme_path) => {
                     theme_path.push("themes");
@@ -390,12 +406,12 @@ impl Model {
                             self.preview_theme_apply(theme, index);
                         }
                         Err(e) => {
-                            error!("Failed to load theme colors: {:?}", e);
+                            error!("Failed to load theme colors: {e:?}");
                         }
                     }
                 }
                 Err(e) => {
-                    error!("Error getting config path: {:?}", e);
+                    error!("Error getting config path: {e:?}");
                 }
             }
         }
