@@ -1,19 +1,171 @@
-use crate::ui::Model;
 use std::path::Path;
+
+use either::Either;
+use termusiclib::config::v2::tui::keys::Keys;
 use termusiclib::config::SharedTuiSettings;
+use termusiclib::ids::Id;
 use termusiclib::library_db::const_unknown::{UNKNOWN_ARTIST, UNKNOWN_FILE, UNKNOWN_TITLE};
-use termusiclib::library_db::{Indexable, SearchCriteria};
-use termusiclib::types::{DBMsg, Id, Msg};
+use termusiclib::library_db::{Indexable, SearchCriteria, TrackDB};
+use termusiclib::track::DurationFmtShort;
+use termusiclib::types::{DBMsg, GSMsg, Msg};
 use termusiclib::utils::{is_playlist, playlist_get_vec};
 use tui_realm_stdlib::List;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::props::Borders;
 use tuirealm::props::{Alignment, BorderType, Table, TableBuilder, TextSpan};
 use tuirealm::{
-    event::{Key, KeyEvent, KeyModifiers, NoUserEvent},
+    event::{Key, KeyEvent, KeyModifiers},
     AttrValue, Attribute, Component, Event, MockComponent, State, StateValue,
 };
 
+use super::popups::{YNConfirm, YNConfirmStyle};
+use crate::ui::model::UserEvent;
+use crate::ui::Model;
+
+/// Helper trait to accomedate mutable access to `self` while also allowing access to other `self` properties for [`common_list_movement`].
+trait OnKeyDB {
+    fn on_key_tab(&self) -> Msg;
+    fn on_key_backtab(&self) -> Msg;
+}
+
+/// Common matches for [`List`] component movement and events
+fn common_list_movement<C: MockComponent + OnKeyDB>(
+    comp: &mut C,
+    keys: &Keys,
+    ev: &Event<UserEvent>,
+) -> Option<Either<CmdResult, Msg>> {
+    let res = match ev {
+        Event::Keyboard(KeyEvent {
+            code: Key::Up,
+            modifiers: KeyModifiers::NONE,
+        }) => comp.perform(Cmd::Move(Direction::Up)),
+        Event::Keyboard(key) if *key == keys.navigation_keys.up.get() => {
+            comp.perform(Cmd::Move(Direction::Up))
+        }
+
+        Event::Keyboard(KeyEvent {
+            code: Key::Down,
+            modifiers: KeyModifiers::NONE,
+        }) => {
+            if let Some(AttrValue::Table(t)) = comp.query(Attribute::Content) {
+                if let State::One(StateValue::Usize(index)) = comp.state() {
+                    if index >= t.len() - 1 {
+                        return Some(Either::Right(comp.on_key_tab()));
+                    }
+                }
+            }
+            comp.perform(Cmd::Move(Direction::Down))
+        }
+        Event::Keyboard(key) if *key == keys.navigation_keys.down.get() => {
+            if let Some(AttrValue::Table(t)) = comp.query(Attribute::Content) {
+                if let State::One(StateValue::Usize(index)) = comp.state() {
+                    if index >= t.len() - 1 {
+                        return Some(Either::Right(comp.on_key_tab()));
+                    }
+                }
+            }
+            comp.perform(Cmd::Move(Direction::Down))
+        }
+
+        Event::Keyboard(KeyEvent {
+            code: Key::PageUp,
+            modifiers: KeyModifiers::NONE,
+        }) => comp.perform(Cmd::Scroll(Direction::Up)),
+        Event::Keyboard(KeyEvent {
+            code: Key::PageDown,
+            modifiers: KeyModifiers::NONE,
+        }) => comp.perform(Cmd::Scroll(Direction::Down)),
+
+        Event::Keyboard(KeyEvent {
+            code: Key::Home,
+            modifiers: KeyModifiers::NONE,
+        }) => comp.perform(Cmd::GoTo(Position::Begin)),
+        Event::Keyboard(key) if *key == keys.navigation_keys.goto_top.get() => {
+            comp.perform(Cmd::GoTo(Position::Begin))
+        }
+
+        Event::Keyboard(KeyEvent {
+            code: Key::End,
+            modifiers: KeyModifiers::NONE,
+        }) => comp.perform(Cmd::GoTo(Position::End)),
+        Event::Keyboard(key) if *key == keys.navigation_keys.goto_bottom.get() => {
+            comp.perform(Cmd::GoTo(Position::End))
+        }
+
+        Event::Keyboard(KeyEvent {
+            code: Key::Tab,
+            modifiers: KeyModifiers::NONE,
+        }) => return Some(Either::Right(comp.on_key_tab())),
+        Event::Keyboard(KeyEvent {
+            code: Key::BackTab,
+            modifiers: KeyModifiers::SHIFT,
+        }) => return Some(Either::Right(comp.on_key_backtab())),
+
+        _ => return None,
+    };
+
+    Some(Either::Left(res))
+}
+
+/// Like [`SearchCriteria`], but specific to TUI and mapping to & from a [`Table`].
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+enum DBCriteria {
+    Artists,
+    Albums,
+    Genres,
+    Directories,
+    Playlists,
+}
+
+impl DBCriteria {
+    /// Number of elements in the table.
+    /// This is for example used to get exact space allocation for the layout.
+    ///
+    /// Note: keep this in-sync with [`Self::build_table`]
+    const NUM_OPTIONS: u16 = 5;
+
+    fn build_table() -> Table {
+        TableBuilder::default()
+            .add_col(TextSpan::from("Artist"))
+            .add_row()
+            .add_col(TextSpan::from("Album"))
+            .add_row()
+            .add_col(TextSpan::from("Genre"))
+            .add_row()
+            .add_col(TextSpan::from("Directory"))
+            .add_row()
+            .add_col(TextSpan::from("Playlists"))
+            .build()
+    }
+
+    /// Try to map the given table index to a variant, returns [`None`] if index is out of bounds.
+    fn from_table_index(idx: usize) -> Option<Self> {
+        // NOTE: this has to match whatever `build_table` produces
+        let res = match idx {
+            0 => Self::Artists,
+            1 => Self::Albums,
+            2 => Self::Genres,
+            3 => Self::Directories,
+            4 => Self::Playlists,
+            _ => return None,
+        };
+
+        Some(res)
+    }
+}
+
+impl From<DBCriteria> for SearchCriteria {
+    fn from(value: DBCriteria) -> Self {
+        match value {
+            DBCriteria::Artists => Self::Artist,
+            DBCriteria::Albums => Self::Album,
+            DBCriteria::Genres => Self::Genre,
+            DBCriteria::Directories => Self::Directory,
+            DBCriteria::Playlists => Self::Playlist,
+        }
+    }
+}
+
 #[derive(MockComponent)]
 pub struct DBListCriteria {
     component: List,
@@ -41,19 +193,7 @@ impl DBListCriteria {
                 .rewind(false)
                 .step(4)
                 .scroll(true)
-                .rows(
-                    TableBuilder::default()
-                        .add_col(TextSpan::from("Artist"))
-                        .add_row()
-                        .add_col(TextSpan::from("Album"))
-                        .add_row()
-                        .add_col(TextSpan::from("Genre"))
-                        .add_row()
-                        .add_col(TextSpan::from("Directory"))
-                        .add_row()
-                        .add_col(TextSpan::from("Playlists"))
-                        .build(),
-                )
+                .rows(DBCriteria::build_table())
         };
 
         Self {
@@ -63,96 +203,93 @@ impl DBListCriteria {
             config,
         }
     }
+
+    /// Get the number of static options in the list.
+    // See `DBCriteria::num_option` for actual implementation.
+    pub const fn num_options() -> u16 {
+        DBCriteria::NUM_OPTIONS
+    }
 }
 
-impl Component<Msg, NoUserEvent> for DBListCriteria {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl OnKeyDB for DBListCriteria {
+    fn on_key_tab(&self) -> Msg {
+        self.on_key_tab.clone()
+    }
+
+    fn on_key_backtab(&self) -> Msg {
+        self.on_key_backtab.clone()
+    }
+}
+
+impl Component<Msg, UserEvent> for DBListCriteria {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
-        let cmd_result = match ev {
-            Event::Keyboard(KeyEvent {
-                code: Key::Down,
-                modifiers: KeyModifiers::NONE,
-            }) => {
-                if let Some(AttrValue::Table(t)) = self.query(Attribute::Content) {
-                    if let State::One(StateValue::Usize(index)) = self.state() {
-                        if index >= t.len() - 1 {
-                            return Some(self.on_key_tab.clone());
-                        }
-                    }
-                }
-                self.perform(Cmd::Move(Direction::Down))
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Up,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::Move(Direction::Up)),
-            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
-                if let Some(AttrValue::Table(t)) = self.query(Attribute::Content) {
-                    if let State::One(StateValue::Usize(index)) = self.state() {
-                        if index >= t.len() - 1 {
-                            return Some(self.on_key_tab.clone());
-                        }
-                    }
-                }
-                self.perform(Cmd::Move(Direction::Down))
-            }
-            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
-                self.perform(Cmd::Move(Direction::Up))
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::PageDown,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::Scroll(Direction::Down)),
-            Event::Keyboard(KeyEvent {
-                code: Key::PageUp,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
-                self.perform(Cmd::GoTo(Position::Begin))
-            }
-            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
-                self.perform(Cmd::GoTo(Position::End))
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Home,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::GoTo(Position::Begin)),
 
-            Event::Keyboard(KeyEvent {
-                code: Key::Enter,
-                modifiers: KeyModifiers::NONE,
-            }) => {
-                if let State::One(StateValue::Usize(index)) = self.state() {
-                    return Some(Msg::DataBase(DBMsg::SearchResult(index)));
+        let cmd_result = common_list_movement(self, keys, &ev).unwrap_or_else(|| {
+            let res = match ev {
+                Event::Keyboard(KeyEvent {
+                    code: Key::Enter,
+                    modifiers: KeyModifiers::NONE,
+                }) => {
+                    if let State::One(StateValue::Usize(index)) = self.state() {
+                        let criteria = DBCriteria::from_table_index(index)
+                            .expect("All table options to be mapped");
+                        return Either::Right(Msg::DataBase(DBMsg::SearchResult(criteria.into())));
+                    }
+                    CmdResult::None
                 }
-                CmdResult::None
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::End,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::GoTo(Position::End)),
-            Event::Keyboard(KeyEvent {
-                code: Key::Tab,
-                modifiers: KeyModifiers::NONE,
-            }) => return Some(self.on_key_tab.clone()),
-            Event::Keyboard(KeyEvent {
-                code: Key::BackTab,
-                modifiers: KeyModifiers::SHIFT,
-            }) => return Some(self.on_key_backtab.clone()),
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
-                return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowDatabase))
-            }
-            _ => CmdResult::None,
-        };
+                Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
+                    return Either::Right(Msg::GeneralSearch(GSMsg::PopupShowDatabase))
+                }
+                _ => CmdResult::None,
+            };
+
+            Either::Left(res)
+        });
+
         match cmd_result {
-            CmdResult::None => None,
-            _ => Some(Msg::ForceRedraw),
+            Either::Left(CmdResult::None) => None,
+            Either::Right(msg) => Some(msg),
+            Either::Left(_) => Some(Msg::ForceRedraw),
         }
     }
 }
 
+/// Component for a "Are you sure you want to add ALL found albums? Y/N" popup
+#[derive(MockComponent)]
+pub struct AddAlbumConfirm {
+    component: YNConfirm,
+}
+
+impl AddAlbumConfirm {
+    pub fn new(config: SharedTuiSettings, criteria: &str) -> Self {
+        let component = YNConfirm::new_with_cb(
+            config,
+            format!(" Are you sure you want to add EVERYTHING from {criteria}? ",),
+            |config| YNConfirmStyle {
+                foreground_color: config.settings.theme.important_popup_foreground(),
+                background_color: config.settings.theme.important_popup_background(),
+                border_color: config.settings.theme.important_popup_border(),
+                title_alignment: Alignment::Left,
+            },
+        );
+
+        Self { component }
+    }
+}
+
+impl Component<Msg, UserEvent> for AddAlbumConfirm {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
+        self.component.on(
+            ev,
+            Msg::DataBase(DBMsg::AddAllResultsToPlaylist),
+            Msg::DataBase(DBMsg::AddAllResultsConfirmCancel),
+        )
+    }
+}
+
 #[derive(MockComponent)]
 pub struct DBListSearchResult {
     component: List,
@@ -196,103 +333,60 @@ impl DBListSearchResult {
     }
 }
 
-impl Component<Msg, NoUserEvent> for DBListSearchResult {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl OnKeyDB for DBListSearchResult {
+    fn on_key_tab(&self) -> Msg {
+        self.on_key_tab.clone()
+    }
+
+    fn on_key_backtab(&self) -> Msg {
+        self.on_key_backtab.clone()
+    }
+}
+
+impl Component<Msg, UserEvent> for DBListSearchResult {
+    #[allow(clippy::too_many_lines)]
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
-        let cmd_result = match ev {
-            Event::Keyboard(KeyEvent {
-                code: Key::Down,
-                modifiers: KeyModifiers::NONE,
-            }) => {
-                if let Some(AttrValue::Table(t)) = self.query(Attribute::Content) {
+
+        let cmd_result = common_list_movement(self, keys, &ev).unwrap_or_else(|| {
+            let res = match ev {
+                Event::Keyboard(KeyEvent {
+                    code: Key::Enter,
+                    modifiers: KeyModifiers::NONE,
+                }) => {
                     if let State::One(StateValue::Usize(index)) = self.state() {
-                        if index >= t.len() - 1 {
-                            return Some(self.on_key_tab.clone());
-                        }
+                        return Either::Right(Msg::DataBase(DBMsg::SearchTrack(index)));
                     }
+                    CmdResult::None
                 }
-                self.perform(Cmd::Move(Direction::Down))
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Up,
-                modifiers: KeyModifiers::NONE,
-            }) => {
-                if let State::One(StateValue::Usize(index)) = self.state() {
-                    if index == 0 {
-                        return Some(self.on_key_backtab.clone());
-                    }
+
+                Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
+                    return Either::Right(Msg::GeneralSearch(GSMsg::PopupShowDatabase));
                 }
-                self.perform(Cmd::Move(Direction::Up))
-            }
-            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
-                if let Some(AttrValue::Table(t)) = self.query(Attribute::Content) {
+
+                Event::Keyboard(keyevent) if keyevent == keys.database_keys.add_selected.get() => {
                     if let State::One(StateValue::Usize(index)) = self.state() {
-                        if index >= t.len() - 1 {
-                            return Some(self.on_key_tab.clone());
-                        }
+                        // Maybe we should also have a popup if it is anything other that "Album"?
+                        // Because things like "Genre" could be *very* big
+                        return Either::Right(Msg::DataBase(DBMsg::AddResultToPlaylist(index)));
                     }
+                    CmdResult::None
                 }
-                self.perform(Cmd::Move(Direction::Down))
-            }
-            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
-                if let State::One(StateValue::Usize(index)) = self.state() {
-                    if index == 0 {
-                        return Some(self.on_key_backtab.clone());
-                    }
+                Event::Keyboard(keyevent) if keyevent == keys.database_keys.add_all.get() => {
+                    return Either::Right(Msg::DataBase(DBMsg::AddAllResultsConfirmShow));
                 }
-                self.perform(Cmd::Move(Direction::Up))
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::PageDown,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::Scroll(Direction::Down)),
-            Event::Keyboard(KeyEvent {
-                code: Key::PageUp,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
-                self.perform(Cmd::GoTo(Position::Begin))
-            }
-            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
-                self.perform(Cmd::GoTo(Position::End))
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Home,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::GoTo(Position::Begin)),
-            Event::Keyboard(KeyEvent {
-                code: Key::End,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::GoTo(Position::End)),
 
-            Event::Keyboard(KeyEvent {
-                code: Key::Enter,
-                modifiers: KeyModifiers::NONE,
-            }) => {
-                if let State::One(StateValue::Usize(index)) = self.state() {
-                    return Some(Msg::DataBase(DBMsg::SearchTrack(index)));
-                }
-                CmdResult::None
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Tab,
-                modifiers: KeyModifiers::NONE,
-            }) => return Some(self.on_key_tab.clone()),
-            Event::Keyboard(KeyEvent {
-                code: Key::BackTab,
-                modifiers: KeyModifiers::SHIFT,
-            }) => return Some(self.on_key_backtab.clone()),
+                _ => CmdResult::None,
+            };
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
-                return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowDatabase))
-            }
+            Either::Left(res)
+        });
 
-            _ => CmdResult::None,
-        };
         match cmd_result {
-            CmdResult::None => None,
-            _ => Some(Msg::ForceRedraw),
+            Either::Left(CmdResult::None) => None,
+            Either::Right(msg) => Some(msg),
+            Either::Left(_) => Some(Msg::ForceRedraw),
         }
     }
 }
@@ -340,88 +434,47 @@ impl DBListSearchTracks {
     }
 }
 
-impl Component<Msg, NoUserEvent> for DBListSearchTracks {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl OnKeyDB for DBListSearchTracks {
+    fn on_key_tab(&self) -> Msg {
+        self.on_key_tab.clone()
+    }
+
+    fn on_key_backtab(&self) -> Msg {
+        self.on_key_backtab.clone()
+    }
+}
+
+impl Component<Msg, UserEvent> for DBListSearchTracks {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
-        let cmd_result = match ev {
-            Event::Keyboard(KeyEvent {
-                code: Key::Down,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::Move(Direction::Down)),
-            Event::Keyboard(KeyEvent {
-                code: Key::Up,
-                modifiers: KeyModifiers::NONE,
-            }) => {
-                if let State::One(StateValue::Usize(index)) = self.state() {
-                    if index == 0 {
-                        return Some(self.on_key_backtab.clone());
+
+        let cmd_result = common_list_movement(self, keys, &ev).unwrap_or_else(|| {
+            let res = match ev {
+                Event::Keyboard(keyevent) if keyevent == keys.database_keys.add_selected.get() => {
+                    if let State::One(StateValue::Usize(index)) = self.state() {
+                        return Either::Right(Msg::DataBase(DBMsg::AddPlaylist(index)));
                     }
+                    CmdResult::None
                 }
-                self.perform(Cmd::Move(Direction::Up))
-            }
-            Event::Keyboard(key) if key == keys.navigation_keys.down.get() => {
-                self.perform(Cmd::Move(Direction::Down))
-            }
-            Event::Keyboard(key) if key == keys.navigation_keys.up.get() => {
-                if let State::One(StateValue::Usize(index)) = self.state() {
-                    if index == 0 {
-                        return Some(self.on_key_backtab.clone());
-                    }
+                Event::Keyboard(keyevent) if keyevent == keys.database_keys.add_all.get() => {
+                    return Either::Right(Msg::DataBase(DBMsg::AddAllToPlaylist));
                 }
-                self.perform(Cmd::Move(Direction::Up))
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::PageDown,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::Scroll(Direction::Down)),
-            Event::Keyboard(KeyEvent {
-                code: Key::PageUp,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::Scroll(Direction::Up)),
-            Event::Keyboard(key) if key == keys.navigation_keys.goto_top.get() => {
-                self.perform(Cmd::GoTo(Position::Begin))
-            }
-            Event::Keyboard(key) if key == keys.navigation_keys.goto_bottom.get() => {
-                self.perform(Cmd::GoTo(Position::End))
-            }
-            Event::Keyboard(KeyEvent {
-                code: Key::Home,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::GoTo(Position::Begin)),
-            Event::Keyboard(KeyEvent {
-                code: Key::End,
-                modifiers: KeyModifiers::NONE,
-            }) => self.perform(Cmd::GoTo(Position::End)),
-            Event::Keyboard(KeyEvent {
-                code: Key::Tab,
-                modifiers: KeyModifiers::NONE,
-            }) => return Some(self.on_key_tab.clone()),
 
-            Event::Keyboard(KeyEvent {
-                code: Key::BackTab,
-                modifiers: KeyModifiers::SHIFT,
-            }) => return Some(self.on_key_backtab.clone()),
-
-            Event::Keyboard(keyevent) if keyevent == keys.database_keys.add_selected.get() => {
-                if let State::One(StateValue::Usize(index)) = self.state() {
-                    return Some(Msg::DataBase(DBMsg::AddPlaylist(index)));
+                Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
+                    return Either::Right(Msg::GeneralSearch(GSMsg::PopupShowDatabase));
                 }
-                CmdResult::None
-            }
-            Event::Keyboard(keyevent) if keyevent == keys.database_keys.add_all.get() => {
-                return Some(Msg::DataBase(DBMsg::AddAllToPlaylist))
-            }
 
-            Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
-                return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowDatabase))
-            }
+                _ => CmdResult::None,
+            };
+
+            Either::Left(res)
+        });
 
-            _ => CmdResult::None,
-        };
         match cmd_result {
-            CmdResult::None => None,
-            _ => Some(Msg::ForceRedraw),
+            Either::Left(CmdResult::None) => None,
+            Either::Right(msg) => Some(msg),
+            Either::Left(_) => Some(Msg::ForceRedraw),
         }
     }
 }
@@ -494,7 +547,7 @@ impl Model {
                 _ => {
                     display_name.clone_from(record);
                 }
-            };
+            }
             if !display_name.is_empty() {
                 if idx > 0 {
                     table.add_row();
@@ -546,7 +599,7 @@ impl Model {
         for record in all_items
             .into_iter()
             .filter_map(std::result::Result::ok)
-            .filter(|p| is_playlist(&p.path().to_string_lossy()))
+            .filter(|p| is_playlist(p.path()))
         {
             let full_path_name = record.path().to_string_lossy().to_string();
             vec.push(full_path_name);
@@ -554,35 +607,71 @@ impl Model {
         vec
     }
 
-    pub fn database_update_search_tracks(&mut self, index: usize) {
-        match self.dw.criteria {
+    /// Find all tracks for the given [`criteria`](SearchCriteria) which matches `val`.
+    ///
+    /// Or for the [`Playlist`](SearchCriteria::Playlist) case, `val` is the path of the playlist.
+    pub fn database_get_tracks_by_criteria(
+        &mut self,
+        criteria: SearchCriteria,
+        val: &str,
+    ) -> Option<Vec<TrackDB>> {
+        match criteria {
             SearchCriteria::Playlist => {
-                if let Some(result) = self.dw.search_results.get(index) {
-                    if let Ok(vec) = playlist_get_vec(result) {
-                        let mut vec_db = Vec::new();
-                        for item in vec {
-                            if let Ok(i) = self.db.get_record_by_path(&item) {
-                                vec_db.push(i);
-                            }
+                let path = Path::new(val);
+                if let Ok(vec) = playlist_get_vec(path) {
+                    let mut vec_db = Vec::with_capacity(vec.len());
+                    for item in vec {
+                        if let Ok(i) = self.db.get_record_by_path(&item) {
+                            vec_db.push(i);
                         }
-                        self.dw.search_tracks = vec_db;
                     }
+                    return Some(vec_db);
                 }
             }
             _ => {
-                if let Ok(vec) = self
-                    .db
-                    .get_record_by_criteria(&self.dw.search_results[index], &self.dw.criteria)
-                {
-                    self.dw.search_tracks = vec;
-                };
+                return self.db.get_record_by_criteria(val, &criteria).ok();
             }
         }
 
+        None
+    }
+
+    /// Update view `Tracks` by populating it with items from the selected `Result`(view) index.
+    pub fn database_update_search_tracks(&mut self, index: usize) {
+        self.dw.search_tracks.clear();
+        let Some(at_index) = self.dw.search_results.get(index).cloned() else {
+            return;
+        };
+
+        let Some(result) = self.database_get_tracks_by_criteria(self.dw.criteria, &at_index) else {
+            return;
+        };
+
+        self.dw.search_tracks = result;
+
         self.database_sync_tracks();
         self.app.active(&Id::DBListSearchTracks).ok();
     }
 
+    /// Add all Results (from view `Result`) to the playlist.
+    pub fn database_add_all_results(&mut self) {
+        self.umount_results_add_confirm_database();
+        if !self.dw.search_results.is_empty() {
+            let mut tracks = Vec::new();
+            // clone once instead every value in every iteration
+            let search_results = self.dw.search_results.clone();
+            for result in search_results {
+                if let Some(mut res) =
+                    self.database_get_tracks_by_criteria(self.dw.criteria, &result)
+                {
+                    tracks.append(&mut res);
+                }
+            }
+
+            self.playlist_add_all_from_db(&tracks);
+        }
+    }
+
     pub fn database_reload(&mut self) {
         assert!(self
             .app
@@ -671,8 +760,7 @@ impl Model {
                 table.add_row();
             }
 
-            let duration =
-                termusiclib::track::Track::duration_formatted_short(&record.meta_duration());
+            let duration = DurationFmtShort(record.meta_duration());
             let duration_string = format!("[{duration:^6.6}]");
 
             table
@@ -696,4 +784,25 @@ impl Model {
         let filtered_music = Model::update_search(&db_tracks, input);
         self.general_search_update_show(Model::build_table(filtered_music));
     }
+
+    /// Mount the [`AddAlbumConfirm`] popup
+    pub fn mount_results_add_confirm_database(&mut self, criteria: SearchCriteria) {
+        self.app
+            .remount(
+                Id::DatabaseAddConfirmPopup,
+                Box::new(AddAlbumConfirm::new(
+                    self.config_tui.clone(),
+                    criteria.as_str(),
+                )),
+                Vec::new(),
+            )
+            .unwrap();
+
+        self.app.active(&Id::DatabaseAddConfirmPopup).unwrap();
+    }
+
+    /// Unmount the [`AddAlbumConfirm`] popup
+    pub fn umount_results_add_confirm_database(&mut self) {
+        let _ = self.app.umount(&Id::DatabaseAddConfirmPopup);
+    }
 }
