@@ -1,19 +1,27 @@
-use crate::ui::model::TermusicLayout;
-use crate::ui::tui_cmd::TuiCmd;
-use crate::ui::Model;
-use anyhow::{anyhow, bail, Result};
-use rand::seq::SliceRandom;
 use std::borrow::Cow;
 use std::ffi::OsString;
 use std::path::Path;
+
+use anyhow::{anyhow, bail, Context as _, Result};
+use rand::seq::IndexedRandom;
+use termusiclib::config::v2::server::LoopMode;
 use termusiclib::config::SharedTuiSettings;
+use termusiclib::ids::Id;
 use termusiclib::library_db::const_unknown::{UNKNOWN_ALBUM, UNKNOWN_ARTIST};
 use termusiclib::library_db::SearchCriteria;
 use termusiclib::library_db::TrackDB;
+use termusiclib::player::playlist_helpers::{
+    PlaylistAddTrack, PlaylistPlaySpecific, PlaylistRemoveTrackIndexed, PlaylistSwapTrack,
+    PlaylistTrackSource,
+};
+use termusiclib::player::{
+    PlaylistAddTrackInfo, PlaylistLoopModeInfo, PlaylistRemoveTrackInfo, PlaylistShuffledInfo,
+    PlaylistSwapInfo,
+};
 use termusiclib::track::Track;
-use termusiclib::types::{GSMsg, Id, Msg, PLMsg};
+use termusiclib::track::{DurationFmtShort, PodcastTrackData};
+use termusiclib::types::{GSMsg, Msg, PLMsg};
 use termusiclib::utils::{filetype_supported, get_parent_folder, is_playlist, playlist_get_vec};
-
 use tui_realm_stdlib::Table;
 use tuirealm::props::Borders;
 use tuirealm::props::{Alignment, BorderType, PropPayload, PropValue, TableBuilder, TextSpan};
@@ -22,10 +30,14 @@ use tuirealm::{
     event::KeyModifiers,
 };
 use tuirealm::{
-    event::{Key, KeyEvent, NoUserEvent},
+    event::{Key, KeyEvent},
     AttrValue, Attribute, Component, Event, MockComponent, State, StateValue,
 };
 
+use crate::ui::model::{TermusicLayout, UserEvent};
+use crate::ui::tui_cmd::{PlaylistCmd, TuiCmd};
+use crate::ui::Model;
+
 #[derive(MockComponent)]
 pub struct Playlist {
     component: Table,
@@ -67,9 +79,9 @@ impl Playlist {
     }
 }
 
-impl Component<Msg, NoUserEvent> for Playlist {
+impl Component<Msg, UserEvent> for Playlist {
     #[allow(clippy::too_many_lines)]
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
         let cmd_result = match ev {
@@ -260,14 +272,27 @@ impl Model {
     }
 
     /// Add a playlist (like m3u) to the playlist.
-    fn playlist_add_playlist(&mut self, current_node: &str) -> Result<()> {
-        let vec = playlist_get_vec(current_node)?;
-        // "add_playlist" will still add all possible things, but return errors for which did not work
-        // so sync still needs to happen
-        let res = self.playlist.add_playlist(&vec);
-        self.player_sync_playlist()?;
-        self.playlist_sync();
-        res?;
+    fn playlist_add_playlist(&mut self, playlist_path: &Path) -> Result<()> {
+        let vec = playlist_get_vec(playlist_path)?;
+
+        let sources = vec
+            .into_iter()
+            .map(|v| {
+                if v.starts_with("http") {
+                    PlaylistTrackSource::Url(v)
+                } else {
+                    PlaylistTrackSource::Path(v)
+                }
+            })
+            .collect();
+
+        self.command(TuiCmd::Playlist(PlaylistCmd::AddTrack(
+            PlaylistAddTrack::new_vec(
+                u64::try_from(self.playback.playlist.len()).unwrap(),
+                sources,
+            ),
+        )));
+
         Ok(())
     }
 
@@ -285,59 +310,82 @@ impl Model {
             .episodes
             .get(episode_index)
             .ok_or_else(|| anyhow!("get episode selected failed."))?;
-        self.playlist.add_episode(episode_selected);
-        self.player_sync_playlist()?;
-        self.playlist_sync();
+
+        let source = PlaylistTrackSource::PodcastUrl(episode_selected.url.clone());
+        self.command(TuiCmd::Playlist(PlaylistCmd::AddTrack(
+            PlaylistAddTrack::new_single(
+                u64::try_from(self.playback.playlist.len()).unwrap(),
+                source,
+            ),
+        )));
+
         Ok(())
     }
 
     /// Add the `current_node`, regardless if it is a Track, dir, playlist, etc.
     ///
     /// See [`Model::playlist_add_episode`] for podcast episode adding
-    pub fn playlist_add(&mut self, current_node: &str) -> Result<()> {
-        let p: &Path = Path::new(&current_node);
-        if !p.exists() {
+    pub fn playlist_add(&mut self, path: &Path) -> Result<()> {
+        if !path.exists() {
             return Ok(());
         }
-        if p.is_dir() {
-            let new_items_vec = Self::library_dir_children(p);
-            // "add_playlist" will still add all possible things, but return errors for which did not work
-            // so sync still needs to happen
-            let res = self.playlist.add_playlist(&new_items_vec);
-            self.player_sync_playlist()?;
-            self.playlist_sync();
-            res?;
+        if path.is_dir() {
+            let new_items_vec = Self::library_dir_children(path);
+
+            let sources = new_items_vec
+                .into_iter()
+                .map(PlaylistTrackSource::Path)
+                .collect();
+
+            self.command(TuiCmd::Playlist(PlaylistCmd::AddTrack(
+                PlaylistAddTrack::new_vec(
+                    u64::try_from(self.playback.playlist.len()).unwrap(),
+                    sources,
+                ),
+            )));
+
             return Ok(());
         }
-        self.playlist_add_item(current_node)?;
+        self.playlist_add_item(path)?;
         self.playlist_sync();
         Ok(())
     }
 
     /// Add a Track or a Playlist to the playlist
-    fn playlist_add_item(&mut self, current_node: &str) -> Result<()> {
-        if is_playlist(current_node) {
-            self.playlist_add_playlist(current_node)?;
+    fn playlist_add_item(&mut self, path: &Path) -> Result<()> {
+        if is_playlist(path) {
+            self.playlist_add_playlist(path)?;
             return Ok(());
         }
-        self.playlist.add_track(&current_node)?;
-        self.player_sync_playlist()?;
+        let source = if path.starts_with("http") {
+            PlaylistTrackSource::Url(path.to_string_lossy().to_string())
+        } else {
+            PlaylistTrackSource::Path(path.to_string_lossy().to_string())
+        };
+
+        self.command(TuiCmd::Playlist(PlaylistCmd::AddTrack(
+            PlaylistAddTrack::new_single(
+                u64::try_from(self.playback.playlist.len()).unwrap(),
+                source,
+            ),
+        )));
+
         Ok(())
     }
 
     /// Add [`TrackDB`] to the playlist
     pub fn playlist_add_all_from_db(&mut self, vec: &[TrackDB]) {
-        let vec2: Vec<&str> = vec.iter().map(|f| f.file.as_str()).collect();
-        // "add_playlist" will still add all possible things, but return errors for which did not work
-        // so sync still needs to happen
-        let res = self.playlist.add_playlist(&vec2);
-        if let Err(e) = self.player_sync_playlist() {
-            self.mount_error_popup(e.context("player sync playlist"));
-        }
-        self.playlist_sync();
-        if let Err(e) = res {
-            self.mount_error_popup(anyhow!(e).context("add all to playlist from database"));
-        }
+        let sources = vec
+            .iter()
+            .map(|f| PlaylistTrackSource::Path(f.file.clone()))
+            .collect();
+
+        self.command(TuiCmd::Playlist(PlaylistCmd::AddTrack(
+            PlaylistAddTrack::new_vec(
+                u64::try_from(self.playback.playlist.len()).unwrap(),
+                sources,
+            ),
+        )));
     }
 
     /// Add random album(s) from the database to the playlist
@@ -366,22 +414,152 @@ impl Model {
         self.playlist_add_all_from_db(&vec);
     }
 
+    /// Handle when a playlist has added a track
+    pub fn handle_playlist_add(&mut self, items: PlaylistAddTrackInfo) -> Result<()> {
+        // piggyback off-of the server side implementation for now by re-parsing everything.
+        self.playback.playlist.add_tracks(
+            PlaylistAddTrack {
+                at_index: items.at_index,
+                tracks: vec![items.trackid],
+            },
+            &self.podcast.db_podcast,
+        )?;
+
+        self.playlist_sync();
+
+        Ok(())
+    }
+
+    /// Handle when a playlist has removed a track
+    pub fn handle_playlist_remove(&mut self, items: &PlaylistRemoveTrackInfo) -> Result<()> {
+        self.playback.playlist.handle_grpc_remove(items)?;
+
+        self.playlist_sync();
+
+        Ok(())
+    }
+
+    /// Handle when a playlist was cleared
+    pub fn handle_playlist_clear(&mut self) {
+        self.playback.playlist.clear();
+
+        self.playlist_sync();
+    }
+
+    /// Handle when the playlist loop-mode was changed
+    pub fn handle_playlist_loopmode(&mut self, loop_mode: &PlaylistLoopModeInfo) -> Result<()> {
+        let as_u8 = u8::try_from(loop_mode.mode).context("Failed to convert u32 to u8")?;
+        let loop_mode =
+            LoopMode::tryfrom_discriminant(as_u8).context("Failed to get LoopMode from u8")?;
+        self.playback.playlist.set_loop_mode(loop_mode);
+        self.config_server.write().settings.player.loop_mode = loop_mode;
+        self.playlist_update_title();
+        // Force a redraw as stream updates are not part of the "tick" event and so cant send "Msg"
+        // but need a redraw because ofthe title change
+        self.force_redraw();
+
+        Ok(())
+    }
+
+    /// Handle when the playlist had swapped some tracks
+    pub fn handle_playlist_swap_tracks(&mut self, swapped_tracks: &PlaylistSwapInfo) -> Result<()> {
+        let index_a = usize::try_from(swapped_tracks.index_a)
+            .context("Failed to convert index_a to usize")?;
+        let index_b = usize::try_from(swapped_tracks.index_b)
+            .context("Failed to convert index_b to usize")?;
+
+        self.playback.playlist.swap(index_a, index_b)?;
+
+        self.playlist_sync();
+
+        Ok(())
+    }
+
+    /// Handle when the playlist has been shuffled and so has new order of tracks
+    pub fn handle_playlist_shuffled(&mut self, shuffled: PlaylistShuffledInfo) -> Result<()> {
+        let playlist_comp_selected_index = self.playlist_get_selected_index();
+        // this might be fragile if there are multiple of the same track in the playlist as there is no unique identifier currently
+        let playlist_track_at_old_file = playlist_comp_selected_index
+            .and_then(|idx| self.playback.playlist.tracks().get(idx))
+            .map(Track::as_track_source);
+
+        self.playback
+            .load_from_grpc(shuffled.tracks, &self.podcast.db_podcast)?;
+        self.playlist_sync();
+
+        if let Some(old_id) = playlist_track_at_old_file {
+            let found_new_index = self
+                .playback
+                .playlist
+                .tracks()
+                .iter()
+                .enumerate()
+                .find(|(_, track)| *track == old_id);
+            if let Some((new_index, _)) = found_new_index {
+                self.playlist_locate(new_index);
+            }
+        }
+
+        Ok(())
+    }
+
+    /// Handle setting the current track index in the TUI playlist and selecting the proper list item
+    ///
+    /// Note: currently this function is called twice per track change, once for `UpdateEvents::TrackChanged` and once for `run_playback::GetProgress`
+    pub fn handle_current_track_index(&mut self, current_track_index: usize, force_relocate: bool) {
+        let tui_old_current_index = self.playback.playlist.current_track_index();
+        info!(
+            "index from player is: {current_track_index:?}, index in tui is: {tui_old_current_index:?}"
+        );
+        self.playback.clear_current_track();
+        let _ = self
+            .playback
+            .playlist
+            .set_current_track_index(current_track_index);
+        self.playback.set_current_track_from_playlist();
+
+        let playlist_comp_selected_index = self.playlist_get_selected_index();
+
+        // only re-select the current-track if the old selection was the old-current-track
+        if force_relocate
+            || (tui_old_current_index.is_some()
+                && playlist_comp_selected_index.is_none_or(|v| v == tui_old_current_index.unwrap()))
+        {
+            self.playlist_locate(current_track_index);
+        }
+
+        self.update_layout_for_current_track();
+        self.player_update_current_track_after();
+
+        self.lyric_update_for_podcast_by_current_track();
+
+        if let Err(e) = self.podcast_mark_current_track_played() {
+            self.mount_error_popup(e.context("Marking podcast track as played"));
+        }
+    }
+
     fn playlist_sync_podcasts(&mut self) {
         let mut table: TableBuilder = TableBuilder::default();
 
-        for (idx, record) in self.playlist.tracks().iter().enumerate() {
+        for (idx, track) in self.playback.playlist.tracks().iter().enumerate() {
             if idx > 0 {
                 table.add_row();
             }
 
-            let duration = record.duration_formatted().to_string();
-            let duration_string = format!("[{duration:^7.7}]");
+            let duration_str = if let Some(dur) = track.duration_str_short() {
+                format!("[{dur:^7.7}]")
+            } else {
+                "[--:--]".to_string()
+            };
 
-            let mut title = record.title().unwrap_or("Unknown Title").to_string();
-            if record.podcast_localfile.is_some() {
+            let mut title = track.title().unwrap_or("Unknown Title").to_string();
+            if track
+                .as_podcast()
+                .is_some_and(PodcastTrackData::has_localfile)
+            {
                 title = format!("[D] {title}");
             }
-            if idx == self.playlist.get_current_track_index() {
+            if Some(idx) == self.playback.playlist.current_track_index() {
                 title = format!(
                     "{}{title}",
                     self.config_tui
@@ -392,12 +570,12 @@ impl Model {
                         .playlist
                         .current_track_symbol
                 );
-            };
+            }
             table
-                .add_col(TextSpan::new(duration_string.as_str()))
+                .add_col(TextSpan::new(duration_str.as_str()))
                 .add_col(TextSpan::new(title).bold());
         }
-        if self.playlist.is_empty() {
+        if self.playback.playlist.is_empty() {
             table.add_col(TextSpan::from("0"));
             table.add_col(TextSpan::from("empty playlist"));
         }
@@ -422,22 +600,27 @@ impl Model {
 
         let mut table: TableBuilder = TableBuilder::default();
 
-        for (idx, record) in self.playlist.tracks().iter().enumerate() {
+        for (idx, track) in self.playback.playlist.tracks().iter().enumerate() {
             if idx > 0 {
                 table.add_row();
             }
 
-            let duration = record.duration_formatted().to_string();
-            let duration_string = format!("[{duration:^7.7}]");
+            let duration_str = if let Some(dur) = track.duration_str_short() {
+                format!("[{dur:^7.7}]")
+            } else {
+                "[--:--]".to_string()
+            };
 
-            let noname_string = "No Name".to_string();
-            let name = record.name().unwrap_or(&noname_string);
-            let artist = record.artist().unwrap_or(UNKNOWN_ARTIST);
-            let mut title: Cow<'_, str> = record.title().unwrap_or(name).into();
-            let album = record.album().unwrap_or(UNKNOWN_ALBUM);
+            let mut title: Cow<'_, str> = track.title().map_or_else(|| track.id_str(), Into::into);
+
+            let artist = track.artist().unwrap_or(UNKNOWN_ARTIST);
+            let album = track
+                .as_track()
+                .and_then(|v| v.album())
+                .unwrap_or(UNKNOWN_ALBUM);
 
             // TODO: is there maybe a better option to do this on-demand instead of the whole playlist; like on draw-time?
-            if idx == self.playlist.get_current_track_index() {
+            if Some(idx) == self.playback.playlist.current_track_index() {
                 title = format!(
                     "{}{title}",
                     self.config_tui
@@ -449,15 +632,15 @@ impl Model {
                         .current_track_symbol
                 )
                 .into();
-            };
+            }
 
             table
-                .add_col(TextSpan::new(duration_string.as_str()))
+                .add_col(TextSpan::new(duration_str.as_str()))
                 .add_col(TextSpan::new(artist).fg(tuirealm::ratatui::style::Color::LightYellow))
                 .add_col(TextSpan::new(title).bold())
                 .add_col(TextSpan::new(album));
         }
-        if self.playlist.is_empty() {
+        if self.playback.playlist.is_empty() {
             table.add_col(TextSpan::from("0"));
             table.add_col(TextSpan::from("empty playlist"));
             table.add_col(TextSpan::from(""));
@@ -476,43 +659,91 @@ impl Model {
         self.playlist_update_title();
     }
 
+    /// Delete a track at `index` from the playlist
     pub fn playlist_delete_item(&mut self, index: usize) {
-        if self.playlist.is_empty() {
+        if self.playback.playlist.is_empty() || index >= self.playback.playlist.len() {
             return;
         }
-        self.playlist.remove(index);
-        if let Err(e) = self.player_sync_playlist() {
-            self.mount_error_popup(e.context("player sync playlist"));
-        }
-        self.playlist_sync();
+
+        let Some(track) = self.playback.playlist.tracks().get(index) else {
+            return;
+        };
+
+        let track_source = track.as_track_source();
+
+        self.command(TuiCmd::Playlist(PlaylistCmd::RemoveTrack(
+            PlaylistRemoveTrackIndexed::new_single(u64::try_from(index).unwrap(), track_source),
+        )));
     }
 
+    /// Clear a entire playlist
     pub fn playlist_clear(&mut self) {
-        self.playlist.clear();
-        if let Err(e) = self.player_sync_playlist() {
-            self.mount_error_popup(e.context("player sync playlist"));
+        if self.playback.playlist.is_empty() {
+            return;
         }
-        self.playlist_sync();
+
+        self.command(TuiCmd::Playlist(PlaylistCmd::Clear));
     }
 
+    /// Shuffle the whole playlist
     pub fn playlist_shuffle(&mut self) {
-        self.playlist.shuffle();
-        if let Err(e) = self.player_sync_playlist() {
-            self.mount_error_popup(e.context("player sync playlist"));
+        self.command(TuiCmd::Playlist(PlaylistCmd::Shuffle));
+    }
+
+    /// Send command to swap 2 indexes. Does nothing if either index is out-of-bounds.
+    ///
+    /// # Panics
+    ///
+    /// if `usize` cannot be converted to `u64`
+    fn playlist_swap(&mut self, index_a: usize, index_b: usize) {
+        let len = self.playback.playlist.tracks().len();
+        if index_a.max(index_b) >= len {
+            error!(
+                "Index out-of-bounds, not executing swap: {}",
+                index_a.max(index_b)
+            );
+            return;
         }
-        self.playlist_sync();
+
+        self.command(TuiCmd::Playlist(PlaylistCmd::SwapTrack(
+            PlaylistSwapTrack {
+                index_a: u64::try_from(index_a).unwrap(),
+                index_b: u64::try_from(index_b).unwrap(),
+            },
+        )));
+    }
+
+    /// Swap the given index upwards, does nothing if out-of-bounds or would result in itself.
+    pub fn playlist_swap_up(&mut self, index: usize) {
+        if index == 0 {
+            return;
+        }
+
+        // always guranteed to be above 0, no saturated necessary
+        self.playlist_swap(index, index - 1);
+    }
+
+    /// Swap the given index downwards, does nothing if out-of-bounds.
+    pub fn playlist_swap_down(&mut self, index: usize) {
+        if index >= self.playback.playlist.len().saturating_sub(1) {
+            return;
+        }
+
+        self.playlist_swap(index, index.saturating_add(1));
     }
 
     pub fn playlist_update_library_delete(&mut self) {
-        self.playlist.remove_deleted_items();
-        if let Err(e) = self.player_sync_playlist() {
-            self.mount_error_popup(e.context("player sync playlist"));
-        }
-        self.playlist_sync();
+        self.command(TuiCmd::Playlist(PlaylistCmd::RemoveDeletedItems));
     }
 
     pub fn playlist_update_title(&mut self) {
-        let duration = self.playlist.tracks().iter().map(Track::duration).sum();
+        let duration = self
+            .playback
+            .playlist
+            .tracks()
+            .iter()
+            .filter_map(Track::duration)
+            .sum();
         let display_symbol = self
             .config_tui
             .read()
@@ -524,8 +755,8 @@ impl Model {
         let loop_mode = self.config_server.read().settings.player.loop_mode;
         let title = format!(
             "\u{2500} Playlist \u{2500}\u{2500}\u{2524} Total {} tracks | {} | Mode: {} \u{251c}\u{2500}",
-            self.playlist.len(),
-            Track::duration_formatted_short(&duration),
+            self.playback.playlist.len(),
+            DurationFmtShort(duration),
             loop_mode.display(display_symbol),
         );
         self.app
@@ -536,19 +767,30 @@ impl Model {
             )
             .ok();
     }
+
+    /// Play the currently selected item in the playlist list
     pub fn playlist_play_selected(&mut self, index: usize) {
-        self.playlist.set_current_track_index(index);
-        if let Err(e) = self.player_sync_playlist() {
-            self.mount_error_popup(e.context("player sync playlist"));
-        }
-        self.command(TuiCmd::PlaySelected);
+        let Some(track) = self.playback.playlist.tracks().get(index) else {
+            error!("Track {index} not in playlist!");
+            return;
+        };
+
+        let track_source = track.as_track_source();
+
+        self.command(TuiCmd::Playlist(PlaylistCmd::PlaySpecific(
+            PlaylistPlaySpecific {
+                track_index: u64::try_from(index).unwrap(),
+                id: track_source,
+            },
+        )));
     }
 
     pub fn playlist_update_search(&mut self, input: &str) {
-        let filtered_music = Model::update_search(self.playlist.tracks(), input);
+        let filtered_music = Model::update_search(self.playback.playlist.tracks(), input);
         self.general_search_update_show(Model::build_table(filtered_music));
     }
 
+    /// Select the given index in the playlist list component
     pub fn playlist_locate(&mut self, index: usize) {
         assert!(self
             .app
@@ -560,16 +802,28 @@ impl Model {
             .is_ok());
     }
 
+    /// Get the current selected index in the playlist list component
+    pub fn playlist_get_selected_index(&self) -> Option<usize> {
+        // the index on a "Table" can be set via "AttrValue::Payload(PropPayload::One(PropValue::Usize(val)))", but reading that is stale
+        // as that value is only read in the "Table", not removed or updated, but "state" is
+        let Ok(State::One(StateValue::Usize(val))) = self.app.state(&Id::Playlist) else {
+            return None;
+        };
+
+        Some(val)
+    }
+
     pub fn playlist_get_random_tracks(&mut self, quantity: u32) -> Vec<TrackDB> {
         let mut result = vec![];
         if let Ok(vec) = self.db.get_all_records() {
             let mut i = 0;
             loop {
-                if let Some(record) = vec.choose(&mut rand::thread_rng()) {
+                if let Some(record) = vec.choose(&mut rand::rng()) {
                     if record.title.contains("Unknown Title") {
                         continue;
                     }
-                    if filetype_supported(&record.file) {
+                    let path = Path::new(&record.file);
+                    if filetype_supported(path) {
                         result.push(record.clone());
                         i += 1;
                         if i > quantity - 1 {
@@ -586,7 +840,7 @@ impl Model {
         let mut result = vec![];
         if let Ok(vec) = self.db.get_all_records() {
             loop {
-                if let Some(v) = vec.choose(&mut rand::thread_rng()) {
+                if let Some(v) = vec.choose(&mut rand::rng()) {
                     if v.album.contains("empty") {
                         continue;
                     }
@@ -632,9 +886,10 @@ impl Model {
 
     /// Save the current playlist as m3u in the given full path
     pub fn playlist_save_m3u(&mut self, filename: &Path) -> Result<()> {
-        self.playlist.save_m3u(filename)?;
+        // TODO: move this to server?
+        self.playback.playlist.save_m3u(filename)?;
 
-        self.library_reload_with_node_focus(Some(&filename.to_string_lossy()));
+        self.library_reload_with_node_focus(Some(filename.to_string_lossy().to_string()));
 
         Ok(())
     }
