@@ -1,25 +1,26 @@
-use crate::ui::Model;
+use std::time::Duration;
+
 use anyhow::{anyhow, bail, Result};
 use percent_encoding::{utf8_percent_encode, NON_ALPHANUMERIC};
 use reqwest::ClientBuilder;
 use sanitize_filename::{sanitize_with_options, Options};
 use serde_json::Value;
-use std::time::Duration;
 use termusiclib::config::SharedTuiSettings;
+use termusiclib::ids::Id;
 use termusiclib::podcast::{download_list, EpData, PodcastFeed, PodcastNoId};
-use termusiclib::track::MediaType;
-use termusiclib::types::{Id, Msg, PCMsg};
+use termusiclib::types::{GSMsg, Msg, PCMsg};
 use tokio::runtime::Handle;
 use tui_realm_stdlib::List;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::props::{Alignment, BorderType, TableBuilder, TextSpan};
 use tuirealm::props::{Borders, PropPayload, PropValue};
 use tuirealm::{
-    event::{Key, KeyEvent, KeyModifiers, NoUserEvent},
+    event::{Key, KeyEvent, KeyModifiers},
     AttrValue, Attribute, Component, Event, MockComponent, State, StateValue,
 };
-// left for debug
-// use std::io::Write;
+
+use crate::ui::model::UserEvent;
+use crate::ui::Model;
 
 #[derive(MockComponent)]
 pub struct FeedsList {
@@ -64,9 +65,9 @@ impl FeedsList {
     }
 }
 
-impl Component<Msg, NoUserEvent> for FeedsList {
+impl Component<Msg, UserEvent> for FeedsList {
     #[allow(clippy::too_many_lines)]
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
         let cmd_result = match ev {
@@ -173,7 +174,7 @@ impl Component<Msg, NoUserEvent> for FeedsList {
             }
 
             Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
-                return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowPodcast))
+                return Some(Msg::GeneralSearch(GSMsg::PopupShowPodcast))
             }
             _ => CmdResult::None,
         };
@@ -227,9 +228,9 @@ impl EpisodeList {
     }
 }
 
-impl Component<Msg, NoUserEvent> for EpisodeList {
+impl Component<Msg, UserEvent> for EpisodeList {
     #[allow(clippy::too_many_lines)]
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
         let cmd_result = match ev {
@@ -341,7 +342,7 @@ impl Component<Msg, NoUserEvent> for EpisodeList {
                 CmdResult::None
             }
             Event::Keyboard(keyevent) if keyevent == keys.library_keys.search.get() => {
-                return Some(Msg::GeneralSearch(crate::ui::GSMsg::PopupShowEpisode))
+                return Some(Msg::GeneralSearch(GSMsg::PopupShowEpisode))
             }
             _ => CmdResult::None,
         };
@@ -920,20 +921,19 @@ impl Model {
         if self.podcast.podcasts.is_empty() {
             return Ok(());
         }
-        if let Some(track) = self.playlist.current_track() {
-            if MediaType::Podcast == track.media_type {
-                if let Some(url) = track.file() {
-                    'outer: for pod in &mut self.podcast.podcasts {
-                        for ep in &mut pod.episodes {
-                            if ep.url == url {
-                                if !ep.played {
-                                    ep.played = true;
-                                    self.podcast
-                                        .db_podcast
-                                        .set_played_status(ep.id, ep.played)?;
-                                }
-                                break 'outer;
+        if let Some(track) = self.playback.current_track() {
+            if let Some(podcast_data) = track.as_podcast() {
+                let url = podcast_data.url();
+                'outer: for pod in &mut self.podcast.podcasts {
+                    for ep in &mut pod.episodes {
+                        if ep.url == url {
+                            if !ep.played {
+                                ep.played = true;
+                                self.podcast
+                                    .db_podcast
+                                    .set_played_status(ep.id, ep.played)?;
                             }
+                            break 'outer;
                         }
                     }
                 }
