@@ -1,37 +1,22 @@
-/**
- * MIT License
- *
- * termusic - Copyright (c) 2021 Larry Hao
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in all
- * copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
- */
-use crate::ui::Model;
+//! SPDX-License-Identifier: MIT
+
+use std::path::Path;
+
 use anyhow::{anyhow, bail, Result};
 use termusiclib::config::{SharedTuiSettings, TuiOverlay};
-use termusiclib::types::{GSMsg, Id, Msg};
+use termusiclib::ids::Id;
+use termusiclib::track::MediaTypes;
+use termusiclib::types::{GSMsg, Msg};
 use tui_realm_stdlib::{Input, Table};
 use tui_realm_treeview::TREE_INITIAL_NODE;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
-use tuirealm::event::{Key, KeyEvent, KeyModifiers, NoUserEvent};
+use tuirealm::event::{Key, KeyEvent, KeyModifiers};
 use tuirealm::props::{Alignment, BorderType, Borders, InputType, TableBuilder, TextSpan};
 use tuirealm::{AttrValue, Attribute, Component, Event, MockComponent, State, StateValue};
 
+use crate::ui::model::UserEvent;
+use crate::ui::Model;
+
 #[derive(MockComponent)]
 pub struct GSInputPopup {
     component: Input,
@@ -74,8 +59,8 @@ impl GSInputPopup {
     }
 }
 
-impl Component<Msg, NoUserEvent> for GSInputPopup {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for GSInputPopup {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Left, ..
@@ -308,8 +293,8 @@ impl GSTablePopup {
     }
 }
 
-impl Component<Msg, NoUserEvent> for GSTablePopup {
-    fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
+impl Component<Msg, UserEvent> for GSTablePopup {
+    fn on(&mut self, ev: Event<UserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
         let cmd_result = match ev {
@@ -437,7 +422,8 @@ impl Model {
                 if let Some(line) = table.get(index) {
                     if let Some(text_span) = line.get(1) {
                         let text = &text_span.content;
-                        self.playlist_add(text)?;
+                        let path = Path::new(text);
+                        self.playlist_add(path)?;
                     }
                 }
             }
@@ -457,8 +443,20 @@ impl Model {
                 if let Some(line) = table.get(result_index) {
                     if let Some(file_name_text_span) = line.get(3) {
                         let file_name = &file_name_text_span.content;
-                        for (idx, item) in self.playlist.tracks().iter().enumerate() {
-                            if item.file() == Some(file_name) {
+                        for (idx, item) in self.playback.playlist.tracks().iter().enumerate() {
+                            // NOTE: i dont know if this should apply to anything other than "track_data"
+                            let lower_matched = match item.inner() {
+                                MediaTypes::Track(track_data) => {
+                                    track_data.path().to_string_lossy() == file_name.as_str()
+                                }
+                                MediaTypes::Radio(radio_track_data) => {
+                                    radio_track_data.url() == file_name
+                                }
+                                MediaTypes::Podcast(podcast_track_data) => {
+                                    podcast_track_data.url() == file_name
+                                }
+                            };
+                            if lower_matched {
                                 index = idx;
                                 matched = true;
                             }
@@ -485,8 +483,20 @@ impl Model {
                 if let Some(line) = table.get(result_index) {
                     if let Some(file_name_text_span) = line.get(3) {
                         let file_name = &file_name_text_span.content;
-                        for (idx, item) in self.playlist.tracks().iter().enumerate() {
-                            if item.file() == Some(file_name) {
+                        for (idx, item) in self.playback.playlist.tracks().iter().enumerate() {
+                            // NOTE: i dont know if this should apply to anything other than "track_data"
+                            let lower_matched = match item.inner() {
+                                MediaTypes::Track(track_data) => {
+                                    track_data.path().to_string_lossy() == file_name.as_str()
+                                }
+                                MediaTypes::Radio(radio_track_data) => {
+                                    radio_track_data.url() == file_name
+                                }
+                                MediaTypes::Podcast(podcast_track_data) => {
+                                    podcast_track_data.url() == file_name
+                                }
+                            };
+                            if lower_matched {
                                 index = idx;
                                 matched = true;
                             }
@@ -503,7 +513,8 @@ impl Model {
 
     pub fn general_search_after_database_add_playlist(&mut self) -> Result<()> {
         let track = self.general_search_get_info(3)?;
-        self.playlist_add(&track)?;
+        let path = Path::new(&track);
+        self.playlist_add(path)?;
         Ok(())
     }
 
