@@ -1,61 +1,47 @@
-mod download_tracker;
-/**
- * MIT License
- *
- * termusic - Copyright (C) 2021 Larry Hao
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in all
- * copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
- */
-mod update;
-mod view;
-mod youtube_options;
+//! SPDX-License-Identifier: MIT
 
-use crate::ui::Application;
-use crate::CombinedSettings;
-use download_tracker::DownloadTracker;
+use std::path::PathBuf;
+use std::str::FromStr;
+use std::sync::atomic::Ordering;
+use std::time::{Duration, Instant};
+
+use anyhow::{anyhow, bail, Context, Result};
+use id3::frame::Lyrics as Id3Lyrics;
 use termusiclib::config::v2::tui::keys::Keys;
 use termusiclib::config::v2::tui::theme::ThemeWrap;
-use termusiclib::library_db::{DataBase, SearchCriteria};
-use termusiclib::types::{Id, Msg, SearchLyricState, YoutubeOptions};
-use termusiclib::xywh;
-
-use termusiclib::track::{MediaType, Track};
-#[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
-use termusiclib::ueberzug::UeInstance;
-
-use anyhow::{anyhow, Result};
-use std::path::PathBuf;
-use std::sync::mpsc::{self, Receiver, Sender};
-use std::time::{Duration, Instant};
 use termusiclib::config::{ServerOverlay, SharedServerSettings, SharedTuiSettings};
+use termusiclib::ids::Id;
 use termusiclib::library_db::TrackDB;
+use termusiclib::library_db::{DataBase, SearchCriteria};
+use termusiclib::player::playlist_helpers::PlaylistTrackSource;
+use termusiclib::player::{PlaylistTracks, RunningStatus};
 use termusiclib::podcast::{db::Database as DBPod, Podcast, PodcastFeed};
+use termusiclib::songtag::lrc::Lyric;
 use termusiclib::songtag::SongTag;
 use termusiclib::taskpool::TaskPool;
+use termusiclib::track::{LyricData, MediaTypesSimple, Track};
+use termusiclib::types::{Msg, YoutubeOptions};
+#[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
+use termusiclib::ueberzug::UeInstance;
 use termusiclib::utils::get_app_config_path;
-use termusicplayback::Playlist;
-use tokio::sync::mpsc::UnboundedSender;
+use termusiclib::xywh;
+use tokio::sync::mpsc::{unbounded_channel, UnboundedReceiver, UnboundedSender};
 use tui_realm_treeview::Tree;
-use tuirealm::event::NoUserEvent;
 use tuirealm::terminal::{CrosstermTerminalAdapter, TerminalBridge};
 
+use super::components::TETrack;
 use super::tui_cmd::TuiCmd;
+use crate::ui::Application;
+use crate::CombinedSettings;
+pub use download_tracker::DownloadTracker;
+pub use user_events::UserEvent;
+
+mod download_tracker;
+mod playlist;
+mod update;
+mod user_events;
+mod view;
+mod youtube_options;
 
 #[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum TermusicLayout {
@@ -133,17 +119,182 @@ pub struct ConfigEditorData {
     pub config_changed: bool,
 }
 
+/// Information about the playback status
+#[derive(Debug, Clone)]
+pub struct Playback {
+    /// The Playlist with all the tracks.
+    pub playlist: playlist::TUIPlaylist,
+    /// The current Running Status like Playing / Paused
+    status: RunningStatus,
+    /// The current track, if there is one. Does not need to be in the playlist.
+    current_track: Option<Track>,
+    current_track_pos: Duration,
+}
+
+impl Playback {
+    fn new() -> Self {
+        Self {
+            playlist: playlist::TUIPlaylist::default(),
+            status: RunningStatus::default(),
+            current_track: None,
+            current_track_pos: Duration::ZERO,
+        }
+    }
+
+    #[must_use]
+    pub fn is_stopped(&self) -> bool {
+        self.status == RunningStatus::Stopped
+    }
+
+    #[must_use]
+    #[expect(dead_code)]
+    pub fn is_paused(&self) -> bool {
+        self.status == RunningStatus::Paused
+    }
+
+    #[must_use]
+    pub fn status(&self) -> RunningStatus {
+        self.status
+    }
+
+    pub fn set_status(&mut self, status: RunningStatus) {
+        self.status = status;
+    }
+
+    #[must_use]
+    pub fn current_track(&self) -> Option<&Track> {
+        self.current_track.as_ref()
+    }
+
+    #[must_use]
+    #[expect(dead_code)]
+    pub fn current_track_mut(&mut self) -> Option<&mut Track> {
+        self.current_track.as_mut()
+    }
+
+    pub fn clear_current_track(&mut self) {
+        self.current_track.take();
+    }
+
+    pub fn set_current_track(&mut self, track: Option<Track>) {
+        self.current_track = track;
+    }
+
+    /// Set the current track from the playlist, if there is one
+    pub fn set_current_track_from_playlist(&mut self) {
+        self.set_current_track(self.playlist.current_track().cloned());
+    }
+
+    pub fn current_track_pos(&self) -> Duration {
+        self.current_track_pos
+    }
+
+    pub fn set_current_track_pos(&mut self, pos: Duration) {
+        self.current_track_pos = pos;
+    }
+
+    /// Load Tracks from a GRPC response.
+    ///
+    /// Returns `(Position, Tracks[])`.
+    ///
+    /// # Errors
+    ///
+    /// - when converting from u64 grpc values to usize fails
+    /// - when there is no track-id
+    /// - when reading a Track from path or podcast database fails
+    pub fn load_from_grpc(
+        &mut self,
+        info: PlaylistTracks,
+        podcast_db: &DBPod,
+    ) -> anyhow::Result<()> {
+        let current_track_index = usize::try_from(info.current_track_index)
+            .context("convert current_track_index(u64) to usize")?;
+        let mut playlist_items = Vec::with_capacity(info.tracks.len());
+
+        for (idx, track) in info.tracks.into_iter().enumerate() {
+            let at_index_usize =
+                usize::try_from(track.at_index).context("convert at_index(u64) to usize")?;
+            // assume / require that the tracks are ordered correctly, if not just log a error for now
+            if idx != at_index_usize {
+                error!("Non-matching \"index\" and \"at_index\"!");
+            }
+
+            // this case should never happen with "termusic-server", but grpc marks them as "optional"
+            let Some(id) = track.id else {
+                bail!("Track does not have a id, which is required to load!");
+            };
+
+            let track = match PlaylistTrackSource::try_from(id)? {
+                PlaylistTrackSource::Path(v) => Track::read_track_from_path(v)?,
+                PlaylistTrackSource::Url(v) => Track::new_radio(&v),
+                PlaylistTrackSource::PodcastUrl(v) => {
+                    let episode = podcast_db.get_episode_by_url(&v)?;
+                    Track::from_podcast_episode(&episode)
+                }
+            };
+
+            playlist_items.push(track);
+        }
+
+        self.playlist.set_tracks(playlist_items);
+
+        // the old server playlist implementation will send `current_track_index: 0`, even if there are not tracks
+        // but the new TUI implementation function "set_current_track_index" will refuse to set anything if the index is out-of-bounds
+        if !self.playlist.is_empty() {
+            self.playlist.set_current_track_index(current_track_index)?;
+        }
+
+        self.set_current_track_from_playlist();
+
+        Ok(())
+    }
+}
+
+#[derive(Debug, Clone, PartialEq)]
+pub struct ExtraLyricData {
+    pub for_track: PathBuf,
+    pub data: LyricData,
+    pub selected_idx: usize,
+}
+
+impl ExtraLyricData {
+    /// Cycle to the next lyric frame and parse it.
+    ///
+    /// Returns `Some(RawLyric)` if found.
+    ///
+    /// # Errors
+    ///
+    /// If there are no frames
+    pub fn cycle_lyric(&mut self) -> Result<Option<&Id3Lyrics>> {
+        if self.data.raw_lyrics.is_empty() {
+            bail!("No lyric frames");
+        }
+
+        self.selected_idx += 1;
+        if self.selected_idx >= self.data.raw_lyrics.len() {
+            self.selected_idx = 0;
+        }
+
+        let raw_lyric = self.data.raw_lyrics.get(self.selected_idx);
+        self.data.parsed_lyrics = raw_lyric.and_then(|v| Lyric::from_str(&v.text).ok());
+
+        Ok(raw_lyric)
+    }
+}
+
+pub type TxToMain = UnboundedSender<Msg>;
+
 pub struct Model {
     /// Indicates that the application must quit
     pub quit: bool,
     /// Tells whether to redraw interface
     pub redraw: bool,
     last_redraw: Instant,
-    pub app: Application<Id, Msg, NoUserEvent>,
+    pub app: Application<Id, Msg, UserEvent>,
     /// Used to draw to terminal
     pub terminal: TerminalBridge<CrosstermTerminalAdapter>,
-    pub tx_to_main: Sender<Msg>,
-    pub rx_to_main: Receiver<Msg>,
+    pub tx_to_main: TxToMain,
+    pub rx_to_main: UnboundedReceiver<Msg>,
     /// Sender for Player Commands
     pub cmd_to_server_tx: UnboundedSender<TuiCmd>,
 
@@ -157,22 +308,18 @@ pub struct Model {
     pub podcast: PodcastWidgetData,
     pub config_editor: ConfigEditorData,
 
-    /// Clone of `playlist.current_track`, but kept around when playlist goes empty but song is still playing
-    pub current_song: Option<Track>,
-    pub tageditor_song: Option<Track>,
-    pub time_pos: Duration,
+    pub tageditor_song: Option<TETrack>,
     pub lyric_line: String,
-    pub playlist: Playlist,
+    pub current_track_lyric: Option<ExtraLyricData>,
+    pub playback: Playback,
 
     #[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
-    pub ueberzug_instance: UeInstance,
+    pub ueberzug_instance: Option<UeInstance>,
     pub viuer_supported: ViuerSupported,
     pub xywh: xywh::Xywh,
 
     youtube_options: YoutubeOptions,
     pub songtag_options: Vec<SongTag>,
-    pub sender_songtag: Sender<SearchLyricState>,
-    pub receiver_songtag: Receiver<SearchLyricState>,
     pub download_tracker: DownloadTracker,
     /// Taskpool to limit number of active network requests
     ///
@@ -209,6 +356,7 @@ fn get_viuer_support() -> ViuerSupported {
 }
 
 impl Model {
+    #[allow(clippy::too_many_lines)]
     pub async fn new(config: CombinedSettings, cmd_to_server_tx: UnboundedSender<TuiCmd>) -> Self {
         let CombinedSettings {
             server: config_server,
@@ -216,20 +364,28 @@ impl Model {
         } = config;
         let path = Self::get_full_path_from_config(&config_server.read());
         // TODO: refactor music library tree to be Paths instead?
-        let tree = Tree::new(Self::library_dir_tree(
-            &path,
-            config_server.read().get_library_scan_depth(),
-        ));
+        let tree = Self::loading_tree();
 
-        let (tx3, rx3): (Sender<SearchLyricState>, Receiver<SearchLyricState>) = mpsc::channel();
+        let viuer_supported = if config_tui.read().cover_features_enabled() {
+            get_viuer_support()
+        } else {
+            ViuerSupported::NotSupported
+        };
+
+        info!("Using viuer protocol {viuer_supported:#?}");
 
-        let viuer_supported = get_viuer_support();
         let db = DataBase::new(&config_server.read()).expect("Open Library Database");
         let db_criteria = SearchCriteria::Artist;
         let terminal = TerminalBridge::new_crossterm().expect("Could not initialize terminal");
 
         #[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
-        let ueberzug_instance = UeInstance::default();
+        let ueberzug_instance = if config_tui.read().cover_features_enabled()
+            && viuer_supported == ViuerSupported::NotSupported
+        {
+            Some(UeInstance::default())
+        } else {
+            None
+        };
         let db_path = get_app_config_path().expect("failed to get podcast db path.");
 
         let db_podcast = DBPod::new(&db_path).expect("error connecting to podcast db.");
@@ -245,9 +401,8 @@ impl Model {
                 .concurrent_downloads_max
                 .get(),
         ));
-        let (tx_to_main, rx_to_main) = mpsc::channel();
+        let (tx_to_main, rx_to_main) = unbounded_channel();
 
-        let playlist = Playlist::new(&config_server).expect("Failed to load playlist");
         let app = Self::init_app(&tree, &config_tui);
 
         // This line is required, in order to show the playing message for the first track
@@ -256,6 +411,16 @@ impl Model {
         let ce_theme = config_tui.read().settings.theme.clone();
         let xywh = xywh::Xywh::from(&config_tui.read().settings.coverart);
 
+        let download_tracker = DownloadTracker::default();
+
+        Self::library_scan(
+            tx_to_main.clone(),
+            download_tracker.clone(),
+            &path,
+            config_server.read().get_library_scan_depth(),
+            None,
+        );
+
         Self {
             app,
             quit: false,
@@ -264,9 +429,7 @@ impl Model {
             terminal,
             config_server,
             config_tui,
-            // current_song: None,
             tageditor_song: None,
-            time_pos: Duration::default(),
             lyric_line: String::new(),
 
             library: MusicLibraryData {
@@ -282,8 +445,6 @@ impl Model {
             #[cfg(all(feature = "cover-ueberzug", not(target_os = "windows")))]
             ueberzug_instance,
             songtag_options: vec![],
-            sender_songtag: tx3,
-            receiver_songtag: rx3,
             viuer_supported,
             db,
             layout: TermusicLayout::TreeView,
@@ -308,10 +469,10 @@ impl Model {
             taskpool,
             tx_to_main,
             rx_to_main,
-            download_tracker: DownloadTracker::default(),
-            playlist,
+            download_tracker,
+            current_track_lyric: None,
+            playback: Playback::new(),
             cmd_to_server_tx,
-            current_song: None,
             xywh,
         }
     }
@@ -350,10 +511,7 @@ impl Model {
     pub fn init_terminal(&mut self) {
         let original_hook = std::panic::take_hook();
         std::panic::set_hook(Box::new(move |panic| {
-            let mut terminal_clone =
-                TerminalBridge::new_crossterm().expect("Could not initialize terminal");
-            let _drop = terminal_clone.disable_raw_mode();
-            let _drop = terminal_clone.leave_alternate_screen();
+            Self::hook_reset_terminal();
             original_hook(panic);
         }));
         let _drop = self.terminal.enable_raw_mode();
@@ -361,51 +519,62 @@ impl Model {
         // required as "enter_alternate_screen" always enabled mouse-capture
         let _drop = self.terminal.disable_mouse_capture();
         let _drop = self.terminal.clear_screen();
+        crate::TERMINAL_ALTERNATE_MODE.store(true, Ordering::SeqCst);
+    }
+
+    /// Finalize terminal for hooks like panic or CTRL+C
+    pub fn hook_reset_terminal() {
+        let mut terminal_clone =
+            TerminalBridge::new_crossterm().expect("Could not initialize terminal");
+        let _drop = terminal_clone.disable_raw_mode();
+        let _drop = terminal_clone.leave_alternate_screen();
+        crate::TERMINAL_ALTERNATE_MODE.store(false, Ordering::SeqCst);
     }
 
     /// Finalize terminal
     pub fn finalize_terminal(&mut self) {
         let _drop = self.terminal.disable_raw_mode();
         let _drop = self.terminal.leave_alternate_screen();
+        crate::TERMINAL_ALTERNATE_MODE.store(false, Ordering::SeqCst);
     }
+
     /// Returns elapsed time since last redraw
     pub fn since_last_redraw(&self) -> Duration {
         self.last_redraw.elapsed()
     }
+
+    /// Force a redraw of the entire model
     pub fn force_redraw(&mut self) {
         self.redraw = true;
     }
 
+    /// Send a command to request the Track Progress and set the titles to the current state.
     pub fn run(&mut self) {
         self.command(TuiCmd::GetProgress);
         self.progress_update_title();
         self.lyric_update_title();
     }
 
-    pub fn player_sync_playlist(&mut self) -> Result<()> {
-        self.playlist.save()?;
-        self.command(TuiCmd::ReloadPlaylist);
-        Ok(())
-    }
-
     pub fn player_update_current_track_after(&mut self) {
-        self.time_pos = Duration::default();
+        self.playback.set_current_track_pos(Duration::ZERO);
         if let Err(e) = self.update_photo() {
             self.mount_error_popup(e.context("update_photo"));
-        };
+        }
         self.progress_update_title();
         self.lyric_update_title();
         self.update_playing_song();
     }
 
+    /// Send a [`TogglePause`](TuiCmd::TogglePause) command, if the conditions are right.
     pub fn player_toggle_pause(&mut self) {
-        if self.playlist.is_empty() && self.playlist.current_track().is_none() {
+        if self.playback.playlist.is_empty() && self.playback.current_track().is_none() {
             return;
         }
 
         self.command(TuiCmd::TogglePause);
     }
 
+    /// Send a [`SkipPrevious`](TuiCmd::SkipPrevious) command.
     pub fn player_previous(&mut self) {
         self.command(TuiCmd::SkipPrevious);
     }
@@ -418,8 +587,8 @@ impl Model {
     }
 
     pub fn is_radio(&self) -> bool {
-        if let Some(track) = self.playlist.current_track() {
-            if track.media_type == MediaType::LiveRadio {
+        if let Some(track) = self.playback.current_track() {
+            if track.media_type() == MediaTypesSimple::LiveRadio {
                 return true;
             }
         }
