@@ -1,42 +1,23 @@
-/*!
- * MIT License
- *
- * termusic - Copyright (C) 2021 Larry Hao
- *
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in all
- * copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
- */
+//! SPDX-License-Identifier: MIT
 
-use crate::ui::tui_cmd::TuiCmd;
-use crate::ui::{model::TermusicLayout, Model};
-use anyhow::anyhow;
-use std::path::Path;
+use std::path::{Path, PathBuf};
 use std::time::Duration;
-use termusiclib::library_db::SearchCriteria;
-use termusiclib::track::MediaType;
+
+use anyhow::anyhow;
+use termusiclib::ids::{Id, IdTagEditor};
+use termusiclib::track::MediaTypesSimple;
 use termusiclib::types::{
-    DBMsg, DLMsg, GSMsg, Id, IdTagEditor, LIMsg, LyricMsg, Msg, PCMsg, PLMsg, XYWHMsg, YSMsg,
+    DBMsg, DLMsg, GSMsg, LIMsg, LyricMsg, MainLayoutMsg, Msg, PCMsg, PLMsg, PlayerMsg,
+    SavePlaylistMsg, XYWHMsg, YSMsg,
 };
 use tokio::runtime::Handle;
 use tokio::time::sleep;
 use tuirealm::props::{AttrValue, Attribute};
 use tuirealm::Update;
 
+use crate::ui::tui_cmd::TuiCmd;
+use crate::ui::{model::TermusicLayout, Model};
+
 impl Update<Msg> for Model {
     #[allow(clippy::too_many_lines)]
     fn update(&mut self, msg: Option<Msg>) -> Option<Msg> {
@@ -46,7 +27,7 @@ impl Update<Msg> for Model {
         // Match message
         match msg {
             Msg::ConfigEditor(m) => self.update_config_editor(m),
-            Msg::DataBase(m) => self.update_database_list(&m),
+            Msg::DataBase(m) => self.update_database_list(m),
 
             Msg::DeleteConfirmShow | Msg::DeleteConfirmCloseCancel | Msg::DeleteConfirmCloseOk => {
                 self.update_delete_confirmation(&msg)
@@ -75,7 +56,7 @@ impl Update<Msg> for Model {
                 None
             }
             Msg::Library(m) => {
-                self.update_library(&m);
+                self.update_library(m);
                 None
             }
             Msg::GeneralSearch(m) => {
@@ -87,14 +68,7 @@ impl Update<Msg> for Model {
                 None
             }
 
-            Msg::PlayerTogglePause
-            | Msg::PlayerToggleGapless
-            | Msg::PlayerSpeedUp
-            | Msg::PlayerSpeedDown
-            | Msg::PlayerVolumeUp
-            | Msg::PlayerVolumeDown
-            | Msg::PlayerSeekForward
-            | Msg::PlayerSeekBackward => self.update_player(&msg),
+            Msg::Player(msg) => self.update_player(msg),
 
             Msg::HelpPopupShow => {
                 self.mount_help_popup();
@@ -120,7 +94,7 @@ impl Update<Msg> for Model {
                 None
             }
             Msg::TagEditor(m) => {
-                self.update_tageditor(&m);
+                self.update_tageditor(m);
                 None
             }
             Msg::UpdatePhoto => {
@@ -129,47 +103,13 @@ impl Update<Msg> for Model {
                 }
                 None
             }
-            Msg::LayoutDataBase | Msg::LayoutTreeView | Msg::LayoutPodCast => {
-                self.update_layout(&msg)
-            }
+            Msg::Layout(msg) => self.update_layout(msg),
+
+            Msg::SavePlaylist(msg) => self.update_save_playlist(msg),
 
-            Msg::SavePlaylistPopupShow => {
-                if let Err(e) = self.mount_save_playlist() {
-                    self.mount_error_popup(e.context("mount save playlist"));
-                }
-                None
-            }
-            Msg::SavePlaylistPopupCloseCancel => {
-                self.umount_save_playlist();
-                None
-            }
-            Msg::SavePlaylistPopupCloseOk(filename) => {
-                self.umount_save_playlist();
-                if let Err(e) = self.playlist_save_m3u_before(&filename) {
-                    self.mount_error_popup(e.context("save m3u playlist before"));
-                }
-                None
-            }
-            Msg::SavePlaylistPopupUpdate(filename) => {
-                if let Err(e) = self.remount_save_playlist_label(&filename) {
-                    self.mount_error_popup(e.context("remount save playlist label"));
-                }
-                None
-            }
-            Msg::SavePlaylistConfirmCloseCancel => {
-                self.umount_save_playlist_confirm();
-                None
-            }
-            Msg::SavePlaylistConfirmCloseOk(filename) => {
-                if let Err(e) = self.playlist_save_m3u(Path::new(&filename)) {
-                    self.mount_error_popup(e.context("save m3u playlist"));
-                }
-                self.umount_save_playlist_confirm();
-                None
-            }
             Msg::Podcast(m) => self.update_podcast(m),
             Msg::LyricMessage(m) => self.update_lyric_textarea(m),
-            Msg::Download(m) => self.update_download_msg(&m),
+            Msg::Download(m) => self.update_download_msg(m),
             Msg::Xywh(m) => self.update_xywh_msg(m),
 
             Msg::ForceRedraw => None,
@@ -178,7 +118,7 @@ impl Update<Msg> for Model {
 }
 
 impl Model {
-    /// Ensure the [`QuitPopup`](crate::ui::components::QuitPopup) always has the focus (top-most)
+    /// Ensure the `QuitPopup` always has the focus (top-most)
     pub fn ensure_quit_popup_top_most_focus(&mut self) {
         if self.app.mounted(&Id::QuitPopup)
             && !self.app.focus().is_some_and(|v| *v == Id::QuitPopup)
@@ -198,7 +138,7 @@ impl Model {
             XYWHMsg::ToggleHidden => {
                 self.xywh_toggle_hide();
             }
-        };
+        }
         None
     }
 
@@ -251,7 +191,7 @@ impl Model {
                 );
                 if let Err(e) = self.add_or_sync_data(&pod, Some(id)) {
                     self.mount_error_popup(e.context("add or sync data"));
-                };
+                }
             }
             PCMsg::NewData(pod) => {
                 self.download_tracker.decrease_one(&pod.url);
@@ -412,12 +352,14 @@ impl Model {
         }
         None
     }
-    fn update_player(&mut self, msg: &Msg) -> Option<Msg> {
+
+    /// Handle Player related messages & events
+    fn update_player(&mut self, msg: PlayerMsg) -> Option<Msg> {
         match msg {
-            Msg::PlayerTogglePause => {
+            PlayerMsg::TogglePause => {
                 self.player_toggle_pause();
             }
-            Msg::PlayerSeekForward => {
+            PlayerMsg::SeekForward => {
                 if self.is_radio() {
                     self.show_message_timeout_label_help(
                         "seek is not available for live radio",
@@ -429,7 +371,7 @@ impl Model {
                 }
                 self.command(TuiCmd::SeekForward);
             }
-            Msg::PlayerSeekBackward => {
+            PlayerMsg::SeekBackward => {
                 if self.is_radio() {
                     self.show_message_timeout_label_help(
                         "seek is not available for live radio",
@@ -441,28 +383,30 @@ impl Model {
                 }
                 self.command(TuiCmd::SeekBackward);
             }
-            Msg::PlayerSpeedUp => {
+            PlayerMsg::SpeedUp => {
                 self.command(TuiCmd::SpeedUp);
             }
-            Msg::PlayerSpeedDown => {
+            PlayerMsg::SpeedDown => {
                 self.command(TuiCmd::SpeedDown);
             }
-            Msg::PlayerVolumeUp => {
+            PlayerMsg::VolumeUp => {
                 self.command(TuiCmd::VolumeUp);
             }
-            Msg::PlayerVolumeDown => {
+            PlayerMsg::VolumeDown => {
                 self.command(TuiCmd::VolumeDown);
             }
-            Msg::PlayerToggleGapless => {
+            PlayerMsg::ToggleGapless => {
                 self.command(TuiCmd::ToggleGapless);
             }
-            _ => {}
         }
+
         None
     }
-    fn update_layout(&mut self, msg: &Msg) -> Option<Msg> {
+
+    /// Switch the main view / layout
+    fn update_layout(&mut self, msg: MainLayoutMsg) -> Option<Msg> {
         match msg {
-            Msg::LayoutDataBase => {
+            MainLayoutMsg::DataBase => {
                 let mut need_to_set_focus = true;
                 if let Ok(Some(AttrValue::Flag(true))) =
                     self.app.query(&Id::DBListCriteria, Attribute::Focus)
@@ -492,9 +436,8 @@ impl Model {
 
                 self.layout = TermusicLayout::DataBase;
                 self.playlist_switch_layout();
-                None
             }
-            Msg::LayoutTreeView => {
+            MainLayoutMsg::TreeView => {
                 let mut need_to_set_focus = true;
                 if let Ok(Some(AttrValue::Flag(true))) =
                     self.app.query(&Id::Playlist, Attribute::Focus)
@@ -514,10 +457,8 @@ impl Model {
 
                 self.layout = TermusicLayout::TreeView;
                 self.playlist_switch_layout();
-                None
             }
-
-            Msg::LayoutPodCast => {
+            MainLayoutMsg::Podcast => {
                 let mut need_to_set_focus = true;
                 if let Ok(Some(AttrValue::Flag(true))) =
                     self.app.query(&Id::Podcast, Attribute::Focus)
@@ -549,12 +490,12 @@ impl Model {
                 self.layout = TermusicLayout::Podcast;
                 self.podcast_sync_feeds_and_episodes();
                 self.playlist_switch_layout();
-                None
             }
-            _ => None,
         }
+
+        None
     }
-    fn update_database_list(&mut self, msg: &DBMsg) -> Option<Msg> {
+    fn update_database_list(&mut self, msg: DBMsg) -> Option<Msg> {
         match msg {
             DBMsg::CriteriaBlurDown | DBMsg::SearchTracksBlurUp => {
                 self.app.active(&Id::DBListSearchResult).ok();
@@ -568,17 +509,17 @@ impl Model {
             DBMsg::SearchResultBlurUp => {
                 self.app.active(&Id::DBListCriteria).ok();
             }
-            DBMsg::SearchResult(index) => {
-                self.dw.criteria = SearchCriteria::from(*index);
+            DBMsg::SearchResult(criteria) => {
+                self.dw.criteria = criteria;
                 self.database_update_search_results();
             }
             DBMsg::SearchTrack(index) => {
-                self.database_update_search_tracks(*index);
+                self.database_update_search_tracks(index);
             }
             DBMsg::AddPlaylist(index) => {
                 if !self.dw.search_tracks.is_empty() {
-                    if let Some(track) = self.dw.search_tracks.get(*index) {
-                        let file = track.file.clone();
+                    if let Some(track) = self.dw.search_tracks.get(index) {
+                        let file = PathBuf::from(&track.file);
                         if let Err(e) = self.playlist_add(&file) {
                             self.mount_error_popup(e.context("playlist add"));
                         }
@@ -589,17 +530,40 @@ impl Model {
                 let db_search_tracks = self.dw.search_tracks.clone();
                 self.playlist_add_all_from_db(&db_search_tracks);
             }
+
+            DBMsg::AddResultToPlaylist(index) => {
+                if let Some(result) = self.dw.search_results.get(index).cloned() {
+                    if let Some(result) =
+                        self.database_get_tracks_by_criteria(self.dw.criteria, &result)
+                    {
+                        self.playlist_add_all_from_db(&result);
+                    }
+                }
+            }
+            DBMsg::AddAllResultsToPlaylist => {
+                self.database_add_all_results();
+            }
+
+            DBMsg::AddAllResultsConfirmShow => {
+                // dont try showing the popup if there is nothing to add
+                if !self.dw.search_results.is_empty() {
+                    self.mount_results_add_confirm_database(self.dw.criteria);
+                }
+            }
+            DBMsg::AddAllResultsConfirmCancel => {
+                self.umount_results_add_confirm_database();
+            }
         }
         None
     }
 
-    fn update_library(&mut self, msg: &LIMsg) {
+    fn update_library(&mut self, msg: LIMsg) {
         match msg {
             LIMsg::TreeBlur => {
                 assert!(self.app.active(&Id::Playlist).is_ok());
             }
             LIMsg::TreeStepInto(path) => {
-                self.library_stepinto(path);
+                self.library_stepinto(&path);
             }
             LIMsg::TreeStepOut => {
                 self.library_stepout();
@@ -623,6 +587,9 @@ impl Model {
                     self.mount_error_popup(e.context("library remove root"));
                 }
             }
+            LIMsg::TreeNodeReady(vec, focus_node) => {
+                self.library_apply_as_tree(vec, focus_node);
+            }
         }
     }
 
@@ -762,12 +729,12 @@ impl Model {
             GSMsg::PopupCloseDatabaseAddPlaylist => {
                 if let Err(e) = self.general_search_after_database_add_playlist() {
                     self.mount_error_popup(e.context("add to playlist from database search"));
-                };
+                }
             }
             GSMsg::PopupCloseEpisodeAddPlaylist => {
                 if let Err(e) = self.general_search_after_episode_add_playlist() {
                     self.mount_error_popup(e.context("add to playlist from episode search"));
-                };
+                }
             }
             GSMsg::PopupCloseOkEpisodeLocate => {
                 if let Err(e) = self.general_search_after_episode_select() {
@@ -818,7 +785,7 @@ impl Model {
                 }
                 if let Err(e) = self.library_delete_node() {
                     self.mount_error_popup(e.context("library delete song"));
-                };
+                }
             }
             _ => {}
         }
@@ -846,7 +813,7 @@ impl Model {
             PLMsg::LoopModeCycle => {
                 self.command(TuiCmd::CycleLoop);
                 self.config_server.write().settings.player.loop_mode =
-                    self.playlist.cycle_loop_mode();
+                    self.playback.playlist.cycle_loop_mode();
                 self.playlist_update_title();
             }
             PLMsg::PlaylistTableBlurDown => match self.layout {
@@ -862,18 +829,10 @@ impl Model {
                 self.player_previous();
             }
             PLMsg::SwapDown(index) => {
-                self.playlist.swap_down(*index);
-                self.playlist_sync();
-                if let Err(e) = self.player_sync_playlist() {
-                    self.mount_error_popup(e.context("playlist sync playlist"));
-                }
+                self.playlist_swap_down(*index);
             }
             PLMsg::SwapUp(index) => {
-                self.playlist.swap_up(*index);
-                self.playlist_sync();
-                if let Err(e) = self.player_sync_playlist() {
-                    self.mount_error_popup(e.context("playlist sync playlist"));
-                }
+                self.playlist_swap_up(*index);
             }
             PLMsg::AddRandomAlbum => {
                 self.playlist_add_random_album();
@@ -893,14 +852,14 @@ impl Model {
 
     // show a popup for playing song
     pub fn update_playing_song(&mut self) {
-        if let Some(track) = self.playlist.current_track() {
+        if let Some(track) = self.playback.current_track() {
             if self.layout == TermusicLayout::Podcast {
                 let title = track.title().unwrap_or("Unknown Episode");
                 self.update_show_message_timeout("Currently Playing", title, None);
                 return;
             }
-            let name = track.name().unwrap_or("Unknown Song");
-            self.update_show_message_timeout("Currently Playing", name, None);
+            let name = track.title().map_or_else(|| track.id_str(), Into::into);
+            self.update_show_message_timeout("Currently Playing", &name, None);
 
             // TODO: is there a better way to update only a single / 2 columns (prev/next) instead of re-doing the whole playist; OR a way to decide at draw-time?
             // sync playlist to update any dynamic parts added to the columns (like current playing symbol)
@@ -933,18 +892,18 @@ impl Model {
     }
 
     pub fn update_layout_for_current_track(&mut self) {
-        if let Some(track) = self.playlist.current_track() {
-            match track.media_type {
-                MediaType::Podcast => {
+        if let Some(track) = self.playback.current_track() {
+            match track.media_type() {
+                MediaTypesSimple::Podcast => {
                     if self.layout == TermusicLayout::Podcast {
                         return;
                     }
-                    self.update_layout(&Msg::LayoutPodCast);
+                    self.update_layout(MainLayoutMsg::Podcast);
                 }
-                MediaType::Music | MediaType::LiveRadio => match self.layout {
+                MediaTypesSimple::Music | MediaTypesSimple::LiveRadio => match self.layout {
                     TermusicLayout::TreeView | TermusicLayout::DataBase => {}
                     TermusicLayout::Podcast => {
-                        self.update_layout(&Msg::LayoutTreeView);
+                        self.update_layout(MainLayoutMsg::TreeView);
                     }
                 },
             }
@@ -959,20 +918,20 @@ impl Model {
     }
 
     // change status bar text to indicate the downloading state
-    fn update_download_msg(&mut self, msg: &DLMsg) -> Option<Msg> {
+    fn update_download_msg(&mut self, msg: DLMsg) -> Option<Msg> {
         self.redraw = true;
         match msg {
             DLMsg::DownloadRunning(url, title) => {
-                self.download_tracker.increase_one(&**url);
+                self.download_tracker.increase_one(&*url);
                 self.show_message_timeout_label_help(
-                    self.download_tracker.message_download_start(title),
+                    self.download_tracker.message_download_start(&title),
                     None,
                     None,
                     None,
                 );
             }
             DLMsg::DownloadSuccess(url) => {
-                self.download_tracker.decrease_one(url);
+                self.download_tracker.decrease_one(&url);
                 self.show_message_timeout_label_help(
                     self.download_tracker.message_download_complete(),
                     None,
@@ -988,13 +947,14 @@ impl Model {
                 if self.download_tracker.visible() {
                     return None;
                 }
-                self.library_reload_with_node_focus(file.as_deref());
+                self.library_reload_with_node_focus(file);
             }
             DLMsg::DownloadErrDownload(url, title, error_message) => {
-                self.download_tracker.decrease_one(url);
+                self.download_tracker.decrease_one(&url);
                 self.mount_error_popup(anyhow!("download failed: {error_message}"));
                 self.show_message_timeout_label_help(
-                    self.download_tracker.message_download_error_response(title),
+                    self.download_tracker
+                        .message_download_error_response(&title),
                     None,
                     None,
                     None,
@@ -1004,17 +964,17 @@ impl Model {
                 self.mount_error_popup(anyhow!("download ok but tag info is not complete."));
                 self.show_message_timeout_label_help(
                     self.download_tracker
-                        .message_download_error_embed_data(title),
+                        .message_download_error_embed_data(&title),
                     None,
                     None,
                     None,
                 );
             }
             DLMsg::MessageShow((title, text)) => {
-                self.mount_message(title, text);
+                self.mount_message(&title, &text);
             }
             DLMsg::MessageHide((title, text)) => {
-                self.umount_message(title, text);
+                self.umount_message(&title, &text);
             }
             DLMsg::FetchPhotoSuccess(image_wrapper) => {
                 self.show_image(&image_wrapper.data).ok();
@@ -1022,7 +982,43 @@ impl Model {
             DLMsg::FetchPhotoErr(err_text) => {
                 self.show_message_timeout_label_help(err_text, None, None, None);
             }
-        };
+        }
+        None
+    }
+
+    /// Handle & update [`SavePlaylistMsg`] related components.
+    fn update_save_playlist(&mut self, msg: SavePlaylistMsg) -> Option<Msg> {
+        match msg {
+            SavePlaylistMsg::PopupShow => {
+                if let Err(e) = self.mount_save_playlist() {
+                    self.mount_error_popup(e.context("mount save playlist"));
+                }
+            }
+            SavePlaylistMsg::PopupCloseCancel => {
+                self.umount_save_playlist();
+            }
+            SavePlaylistMsg::PopupCloseOk(filename) => {
+                self.umount_save_playlist();
+                if let Err(e) = self.playlist_save_m3u_before(&filename) {
+                    self.mount_error_popup(e.context("save m3u playlist before"));
+                }
+            }
+            SavePlaylistMsg::PopupUpdate(filename) => {
+                if let Err(e) = self.remount_save_playlist_label(&filename) {
+                    self.mount_error_popup(e.context("remount save playlist label"));
+                }
+            }
+            SavePlaylistMsg::ConfirmCloseCancel => {
+                self.umount_save_playlist_confirm();
+            }
+            SavePlaylistMsg::ConfirmCloseOk(filename) => {
+                if let Err(e) = self.playlist_save_m3u(Path::new(&filename)) {
+                    self.mount_error_popup(e.context("save m3u playlist"));
+                }
+                self.umount_save_playlist_confirm();
+            }
+        }
+
         None
     }
 }
